@@ -33,26 +33,31 @@
 #include <QMessageBox>
 #include <QCommandLineParser>
 #include <QSocketNotifier>
+#include <QScreen>
+#include <QWindow>
+#include <QFileSystemWatcher>
+
 #include <gio/gio.h>
 #include <sys/socket.h>
 
+#include <libfm-qt/mountoperation.h>
+#include <libfm-qt/filesearchdialog.h>
+#include <libfm-qt/path.h>
+#include <libfm-qt/terminal.h>
+
 #include "applicationadaptor.h"
 #include "preferencesdialog.h"
 #include "desktoppreferencesdialog.h"
-#include "mountoperation.h"
 #include "autorundialog.h"
 #include "launcher.h"
-#include "filesearchdialog.h"
-
-#include <QScreen>
-#include <QWindow>
+#include "xdgdir.h"
+#include "connectserverdialog.h"
 
 #include <X11/Xlib.h>
 
-#include "xdgdir.h"
-#include <QFileSystemWatcher>
 
-using namespace PCManFM;
+namespace PCManFM {
+
 static const char* serviceName = "org.pcmanfm.PCManFM";
 static const char* ifaceName = "org.pcmanfm.Application";
 
@@ -69,17 +74,19 @@ Application::Application(int& argc, char** argv):
   settings_(),
   profileName_("default"),
   daemonMode_(false),
-  desktopWindows_(),
   enableDesktopManager_(false),
+  desktopWindows_(),
   preferencesDialog_(),
+  editBookmarksialog_(),
   volumeMonitor_(NULL),
   userDirsWatcher_(NULL),
-  lxqtRunning_(false),
-  editBookmarksialog_() {
+  lxqtRunning_(false) {
 
   argc_ = argc;
   argv_ = argv;
 
+  setApplicationVersion(PCMANFM_QT_VERSION);
+
   // QDBusConnection::sessionBus().registerObject("/org/pcmanfm/Application", this);
   QDBusConnection dbus = QDBusConnection::sessionBus();
   if(dbus.registerService(serviceName)) {
@@ -186,8 +193,7 @@ bool Application::parseCommandLineArgs() {
   QCommandLineOption setWallpaperOption(QStringList() << "w" << "set-wallpaper", tr("Set desktop wallpaper from image FILE"), tr("FILE"));
   parser.addOption(setWallpaperOption);
 
-  // don't translate list of modes in description, please
-  QCommandLineOption wallpaperModeOption("wallpaper-mode", tr("Set mode of desktop wallpaper. MODE=(color|stretch|fit|center|tile)"), tr("MODE"));
+  QCommandLineOption wallpaperModeOption("wallpaper-mode", tr("Set mode of desktop wallpaper. MODE=(%1)").arg("color|stretch|fit|center|tile|zoom"), tr("MODE"));
   parser.addOption(wallpaperModeOption);
 
   QCommandLineOption showPrefOption("show-pref", tr("Open Preferences dialog on the page with the specified name"), tr("NAME"));
@@ -338,7 +344,7 @@ void Application::onUserDirsChanged()
       }
     } else {
         qWarning("Application::onUserDirsChanged: %s doesn't exist",
-                    qUtf8Printable(userDesktopFolder_));
+                    userDesktopFolder_.toUtf8().constData());
     }
   }
 }
@@ -350,11 +356,10 @@ void Application::onAboutToQuit() {
 
 bool Application::eventFilter(QObject* watched, QEvent* event) {
   if(watched == desktop()) {
-    switch(event->type()) {
-      case QEvent::StyleChange:
-      case QEvent::ThemeChange:
+    if (event->type() == QEvent::StyleChange ||
+        event->type() == QEvent::ThemeChange) {
         setStyle(new ProxyStyle());
-    };
+    }
   }
   return QObject::eventFilter(watched, event);
 }
@@ -437,11 +442,21 @@ void Application::desktopPrefrences(QString page) {
 void Application::onFindFileAccepted() {
   Fm::FileSearchDialog* dlg = static_cast<Fm::FileSearchDialog*>(sender());
   Fm::Path uri = dlg->searchUri();
-  // FIXME: we should be able to open it in an existing window
-  FmPathList* paths = fm_path_list_new();
-  fm_path_list_push_tail(paths, uri.data());
-  Launcher(NULL).launchPaths(NULL, paths);
-  fm_path_list_unref(paths);
+  Fm::PathList paths;
+  paths.pushTail(uri);
+  MainWindow* window = MainWindow::lastActive();
+  Launcher(window).launchPaths(NULL, paths);
+}
+
+void Application::onConnectToServerAccepted() {
+  ConnectServerDialog* dlg = static_cast<ConnectServerDialog*>(sender());
+  QString uri = dlg->uriText();
+  Fm::Path path = Fm::Path::newForStr(uri.toUtf8().constData());
+  qDebug() << uri << " => " << path.toStr();
+  Fm::PathList paths;
+  paths.pushTail(path);
+  MainWindow* window = MainWindow::lastActive();
+  Launcher(window).launchPaths(NULL, paths);
 }
 
 void Application::findFiles(QStringList paths) {
@@ -452,51 +467,54 @@ void Application::findFiles(QStringList paths) {
   dlg->show();
 }
 
+void Application::connectToServer() {
+  ConnectServerDialog* dlg = new ConnectServerDialog();
+  connect(dlg, &QDialog::accepted, this, &Application::onConnectToServerAccepted);
+  dlg->setAttribute(Qt::WA_DeleteOnClose);
+  dlg->show();
+}
+
 void Application::launchFiles(QString cwd, QStringList paths, bool inNewWindow) {
-  FmPathList* pathList = fm_path_list_new();
-  FmPath* cwd_path = NULL;
+  Fm::PathList pathList;
+  Fm::Path cwd_path;
   QStringList::iterator it;
   Q_FOREACH(const QString& it, paths) {
     QByteArray pathName = it.toLocal8Bit();
-    FmPath* path = NULL;
+    Fm::Path path;
     if(pathName[0] == '/') // absolute path
-        path = fm_path_new_for_path(pathName.constData());
+        path = Fm::Path::newForPath(pathName.constData());
     else if(pathName.contains(":/")) // URI
-        path = fm_path_new_for_uri(pathName.constData());
+        path = Fm::Path::newForUri(pathName.constData());
     else if(pathName == "~") // special case for home dir
-        path = fm_path_ref(fm_path_get_home());
+        path = Fm::Path::getHome();
     else // basename
     {
         if(Q_UNLIKELY(!cwd_path))
-            cwd_path = fm_path_new_for_str(cwd.toLocal8Bit().constData());
-        path = fm_path_new_relative(cwd_path, pathName.constData());
+            cwd_path = Fm::Path::newForStr(cwd.toLocal8Bit().constData());
+        path = cwd_path.newRelative(pathName.constData());
     }
     fm_path_list_push_tail(pathList, path);
-    fm_path_unref(path);
   }
-  if(cwd_path)
-      fm_path_unref(cwd_path);
 
   Launcher(NULL).launchPaths(NULL, pathList);
-  fm_path_list_unref(pathList);
 }
 
-void Application::openFolders(FmFileInfoList* files) {
+void Application::openFolders(Fm::FileInfoList files) {
   Launcher(NULL).launchFiles(NULL, files);
 }
 
-void Application::openFolderInTerminal(FmPath* path) {
+void Application::openFolderInTerminal(Fm::Path path) {
   if(!settings_.terminal().isEmpty()) {
     char* cwd_str;
-    if(fm_path_is_native(path))
-      cwd_str = fm_path_to_str(path);
+    if(path.isNative())
+      cwd_str = path.toStr();
     else { // gio will map remote filesystems to local FUSE-mounted paths here.
-      GFile* gf = fm_path_to_gfile(path);
+      GFile* gf = path.toGfile();
       cwd_str = g_file_get_path(gf);
       g_object_unref(gf);
     }
     GError* err = NULL;
-    if(!fm_terminal_launch(cwd_str, &err)) {
+    if(!Fm::Terminal::launch(cwd_str, &err)) {
       QMessageBox::critical(NULL, tr("Error"), QString::fromUtf8(err->message));
       g_error_free(err);
     }
@@ -515,7 +533,7 @@ void Application::preferences(QString page) {
     preferencesDialog_ = new PreferencesDialog(page);
   }
   else {
-    // TODO: set page
+    preferencesDialog_.data()->selectPage(page);
   }
   preferencesDialog_.data()->show();
   preferencesDialog_.data()->raise();
@@ -534,9 +552,12 @@ void Application::setWallpaper(QString path, QString modeString) {
     }
   }
   // convert mode string to value
-  for(int i = 0; i < G_N_ELEMENTS(valid_wallpaper_modes); ++i) {
+  for(std::size_t i = 0; i < G_N_ELEMENTS(valid_wallpaper_modes); ++i) {
     if(modeString == valid_wallpaper_modes[i]) {
-      mode = (DesktopWindow::WallpaperMode)i;
+      // We don't take safety checks because valid_wallpaper_modes[] is
+      // defined in this function and we can clearly see that it does not
+      // overflow.
+      mode = static_cast<DesktopWindow::WallpaperMode>(i);
       if(mode != settings_.wallpaperMode())
         changed = true;
       break;
@@ -820,7 +841,7 @@ void Application::installSigtermHandler() {
     struct sigaction action;
     action.sa_handler = sigtermHandler;
     ::sigemptyset(&action.sa_mask);
-    action.sa_flags |= SA_RESTART;
+    action.sa_flags = SA_RESTART;
     if(::sigaction(SIGTERM, &action, 0) != 0) {
       qWarning("Couldn't install SIGTERM handler");
     }
@@ -838,3 +859,5 @@ void Application::onSigtermNotified() {
     notifier->setEnabled(true);
   }
 }
+
+} // namespace PCManFM
