@@ -23,21 +23,21 @@
 
 #include <QApplication>
 #include "settings.h"
-#include "libfmqt.h"
-#include "editbookmarksdialog.h"
+#include <libfm-qt/libfmqt.h>
+#include <libfm-qt/editbookmarksdialog.h>
 #include <QVector>
 #include <QPointer>
 #include <QProxyStyle>
 #include <QTranslator>
 #include <gio/gio.h>
 
-
 class QScreen;
 
 class QFileSystemWatcher;
 
 namespace PCManFM {
 
+class MainWindow;
 class DesktopWindow;
 class PreferencesDialog;
 class DesktopPreferencesDialog;
@@ -77,6 +77,7 @@ public:
   void editBookmarks();
   void desktopManager(bool enabled);
   void findFiles(QStringList paths = QStringList());
+  void connectToServer();
 
   bool desktopManagerEnabled() {
     return enableDesktopManager_;
@@ -85,8 +86,8 @@ public:
   void updateFromSettings();
   void updateDesktopsFromSettings();
 
-  void openFolderInTerminal(FmPath* path);
-  void openFolders(FmFileInfoList* files);
+  void openFolderInTerminal(Fm::Path path);
+  void openFolders(Fm::FileInfoList files);
 
   QString profileName() {
     return profileName_;
@@ -108,6 +109,7 @@ protected Q_SLOTS:
   void reloadDesktopsAsNeeded();
 
   void onFindFileAccepted();
+  void onConnectToServerAccepted();
 
 protected:
   virtual bool eventFilter(QObject* watched, QEvent* event);
