@@ -31,7 +31,9 @@
 #include <QDebug>
 #include <QStandardPaths>
 
-using namespace PCManFM;
+namespace PCManFM {
+
+static int iconSizes[] = {96, 72, 64, 48, 36, 32, 24, 20};
 
 DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFlags f):
   QDialog(parent, f),
@@ -52,6 +54,7 @@ DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFl
   ui.wallpaperMode->addItem(tr("Stretch to fit the screen"), DesktopWindow::WallpaperFit);
   ui.wallpaperMode->addItem(tr("Center on the screen"), DesktopWindow::WallpaperCenter);
   ui.wallpaperMode->addItem(tr("Tile the image to fill the entire screen"), DesktopWindow::WallpaperTile);
+  ui.wallpaperMode->addItem(tr("Zoom the image to fill the entire screen"), DesktopWindow::WallpaperZoom);
   int i;
   switch(settings.wallpaperMode()) {
     case DesktopWindow::WallpaperNone:
@@ -69,6 +72,9 @@ DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFl
     case DesktopWindow::WallpaperTile:
       i = 4;
       break;
+    case DesktopWindow::WallpaperZoom:
+      i = 5;
+      break;
     default:
       i = 0;
   }
@@ -78,6 +84,13 @@ DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFl
   qDebug("wallpaper: %s", settings.wallpaper().toUtf8().data());
   ui.imageFile->setText(settings.wallpaper());
 
+  for(std::size_t i = 0; i < G_N_ELEMENTS(iconSizes); ++i) {
+    int size = iconSizes[i];
+    ui.iconSize->addItem(QString("%1 x %1").arg(size), size);
+    if(settings.desktopIconSize() == size)
+      ui.iconSize->setCurrentIndex(i);
+  }
+
   ui.font->setFont(settings.desktopFont());
 
   ui.backgroundColor->setColor(settings.desktopBgColor());
@@ -87,6 +100,10 @@ DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFl
 
   connect(ui.buttonBox->button(QDialogButtonBox::Apply), &QPushButton::clicked,
           this, &DesktopPreferencesDialog::onApplyClicked);
+
+  ui.hMargin->setValue(settings.desktopCellMargins().width());
+  ui.vMargin->setValue(settings.desktopCellMargins().height());
+  connect(ui.lockMargins, &QAbstractButton::clicked, this, &DesktopPreferencesDialog::lockMargins);
 }
 
 DesktopPreferencesDialog::~DesktopPreferencesDialog() {
@@ -108,6 +125,16 @@ void DesktopPreferencesDialog::setupDesktopFolderUi()
     this, &DesktopPreferencesDialog::onBrowseDesktopFolderClicked);
 }
 
+void DesktopPreferencesDialog::lockMargins(bool lock) {
+  ui.vMargin->setDisabled(lock);
+  if(lock) {
+    ui.vMargin->setValue(ui.hMargin->value());
+    connect(ui.hMargin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), ui.vMargin, &QSpinBox::setValue);
+  }
+  else
+    disconnect(ui.hMargin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), ui.vMargin, &QSpinBox::setValue);
+}
+
 void DesktopPreferencesDialog::applySettings()
 {
   Settings& settings = static_cast<Application*>(qApp)->settings();
@@ -118,11 +145,13 @@ void DesktopPreferencesDialog::applySettings()
   settings.setWallpaper(ui.imageFile->text());
   int mode = ui.wallpaperMode->itemData(ui.wallpaperMode->currentIndex()).toInt();
   settings.setWallpaperMode(mode);
+  settings.setDesktopIconSize(ui.iconSize->itemData(ui.iconSize->currentIndex()).toInt());
   settings.setDesktopFont(ui.font->font());
   settings.setDesktopBgColor(ui.backgroundColor->color());
   settings.setDesktopFgColor(ui.textColor->color());
   settings.setDesktopShadowColor(ui.shadowColor->color());
   settings.setShowWmMenu(ui.showWmMenu->isChecked());
+  settings.setDesktopCellMargins(QSize(ui.hMargin->value(), ui.vMargin->value()));
 
   settings.save();
 }
@@ -197,3 +226,5 @@ void DesktopPreferencesDialog::setEditDesktopFolder(const bool enabled)
   if (editDesktopFolderEnabled)
       setupDesktopFolderUi();
 }
+
+} // namespace PCManFM
