@@ -26,7 +26,7 @@
 #include <QFile>
 #include <QPixmap>
 #include <QPalette>
-#include <QBrush>
+
 #include <QLayout>
 #include <QDebug>
 #include <QTimer>
@@ -36,18 +36,20 @@
 #include <QShortcut>
 #include <QDropEvent>
 #include <QMimeData>
+#include <QPaintEvent>
 
 #include "./application.h"
 #include "mainwindow.h"
 #include "desktopitemdelegate.h"
-#include "foldermenu.h"
-#include "filemenu.h"
-#include "cachedfoldermodel.h"
-#include "folderview_p.h"
-#include "fileoperation.h"
-#include "filepropsdialog.h"
-#include "utilities.h"
-#include "path.h"
+#include <libfm-qt/foldermenu.h>
+#include <libfm-qt/filemenu.h>
+#include <libfm-qt/cachedfoldermodel.h>
+#include <libfm-qt/folderview_p.h>
+#include <libfm-qt/fileoperation.h>
+#include <libfm-qt/filepropsdialog.h>
+#include <libfm-qt/utilities.h>
+#include <libfm-qt/path.h>
+#include <libfm-qt/fileinfo.h>
 #include "xdgdir.h"
 
 #include <QX11Info>
@@ -55,17 +57,16 @@
 #include <xcb/xcb.h>
 #include <X11/Xlib.h>
 
-using namespace PCManFM;
+namespace PCManFM {
 
 DesktopWindow::DesktopWindow(int screenNum):
   View(Fm::FolderView::IconMode),
-  screenNum_(screenNum),
-  folder_(NULL),
-  model_(NULL),
   proxyModel_(NULL),
+  model_(NULL),
+  wallpaperMode_(WallpaperNone),
   fileLauncher_(NULL),
   showWmMenu_(false),
-  wallpaperMode_(WallpaperNone),
+  screenNum_(screenNum),
   relayoutTimer_(NULL) {
 
   QDesktopWidget* desktopWidget = QApplication::desktop();
@@ -77,10 +78,17 @@ DesktopWindow::DesktopWindow(int screenNum):
   View::setFileLauncher(&fileLauncher_);
 
   listView_ = static_cast<Fm::FolderViewListView*>(childView());
-  listView_->setMovement(QListView::Free);
+  listView_->setMovement(QListView::Snap);
   listView_->setResizeMode(QListView::Adjust);
   listView_->setFlow(QListView::TopToBottom);
 
+  // This is to workaround Qt bug 54384 which affects Qt >= 5.6
+  // https://bugreports.qt.io/browse/QTBUG-54384
+  // Setting a QPixmap larger then the screen resolution to the background of a list view won't work.
+  // So we did a hack here: Disable the automatic background painting.
+  // Then paint the background of the list view ourselves by hook into its paint event handling method with a event filter.
+  listView_->viewport()->setAutoFillBackground(false);
+
   // NOTE: When XRnadR is in use, the all screens are actually combined to form a
   // large virtual desktop and only one DesktopWindow needs to be created and screenNum is -1.
   // In some older multihead setups, such as xinerama, every physical screen
@@ -90,18 +98,21 @@ DesktopWindow::DesktopWindow(int screenNum):
     loadItemPositions();
     Settings& settings = static_cast<Application* >(qApp)->settings();
 
-    model_ = Fm::CachedFolderModel::modelFromPath(fm_path_get_desktop());
-    folder_ = reinterpret_cast<FmFolder*>(g_object_ref(model_->folder()));
+    model_ = Fm::CachedFolderModel::modelFromPath(Fm::Path::getDesktop());
+    folder_ = model_->folder();
 
     proxyModel_ = new Fm::ProxyFolderModel();
     proxyModel_->setSourceModel(model_);
     proxyModel_->setShowThumbnails(settings.showThumbnails());
-    proxyModel_->sort(Fm::FolderModel::ColumnFileMTime);
+    proxyModel_->sort(settings.desktopSortColumn(), settings.desktopSortOrder());
+    proxyModel_->setFolderFirst(settings.desktopSortFolderFirst());
     setModel(proxyModel_);
 
     connect(proxyModel_, &Fm::ProxyFolderModel::rowsInserted, this, &DesktopWindow::onRowsInserted);
     connect(proxyModel_, &Fm::ProxyFolderModel::rowsAboutToBeRemoved, this, &DesktopWindow::onRowsAboutToBeRemoved);
     connect(proxyModel_, &Fm::ProxyFolderModel::layoutChanged, this, &DesktopWindow::onLayoutChanged);
+    connect(proxyModel_, &Fm::ProxyFolderModel::sortFilterChanged, this, &DesktopWindow::onModelSortFilterChanged);
+    connect(proxyModel_, &Fm::ProxyFolderModel::dataChanged, this, &DesktopWindow::onDataChanged);
     connect(listView_, &QListView::indexesMoved, this, &DesktopWindow::onIndexesMoved);
   }
 
@@ -118,6 +129,7 @@ DesktopWindow::DesktopWindow(int screenNum):
   connect(this, &DesktopWindow::openDirRequested, this, &DesktopWindow::onOpenDirRequested);
 
   listView_->installEventFilter(this);
+  listView_->viewport()->installEventFilter(this);
 
   // setup shortcuts
   QShortcut* shortcut;
@@ -147,6 +159,7 @@ DesktopWindow::DesktopWindow(int screenNum):
 }
 
 DesktopWindow::~DesktopWindow() {
+  listView_->viewport()->removeEventFilter(this);
   listView_->removeEventFilter(this);
 
   if(relayoutTimer_)
@@ -157,9 +170,6 @@ DesktopWindow::~DesktopWindow() {
 
   if(model_)
     model_->unref();
-
-  if(folder_)
-    g_object_unref(folder_);
 }
 
 void DesktopWindow::setBackground(const QColor& color) {
@@ -199,7 +209,7 @@ void DesktopWindow::resizeEvent(QResizeEvent* event) {
 }
 
 void DesktopWindow::setDesktopFolder() {
-  FmPath *path = fm_path_new_for_path(XdgDir::readDesktopDir().toStdString().c_str());
+  Fm::Path path = Fm::Path::newForPath(XdgDir::readDesktopDir().toStdString().c_str());
   model_ = Fm::CachedFolderModel::modelFromPath(path);
   proxyModel_->setSourceModel(model_);
 }
@@ -285,14 +295,7 @@ QImage DesktopWindow::loadWallpaperFile(QSize requiredSize) {
 
 // really generate the background pixmap according to current settings and apply it.
 void DesktopWindow::updateWallpaper() {
-  // reset the brush
-  // QPalette palette(listView_->palette());
-  QPalette palette(Fm::FolderView::palette());
-
-  if(wallpaperMode_ == WallpaperNone) { // use background color only
-    palette.setBrush(QPalette::Base, bgColor_);
-  }
-  else { // use wallpaper
+  if(wallpaperMode_ != WallpaperNone) {  // use wallpaper
     QPixmap pixmap;
     QImage image;
     if(wallpaperMode_ == WallpaperTile) { // use the original size
@@ -307,12 +310,13 @@ void DesktopWindow::updateWallpaper() {
       if(wallpaperMode_ == WallpaperCenter) {
         image = QImage(wallpaperFile_); // load original image
       }
-      else if(wallpaperMode_ == WallpaperFit) {
+      else if(wallpaperMode_ == WallpaperFit || wallpaperMode_ == WallpaperZoom) {
         // calculate the desired size
         QSize origSize = QImageReader(wallpaperFile_).size(); // get the size of the original file
         if(origSize.isValid()) {
           QSize desiredSize = origSize;
-          desiredSize.scale(width(), height(), Qt::KeepAspectRatio);
+          Qt::AspectRatioMode mode = (wallpaperMode_ == WallpaperFit ? Qt::KeepAspectRatio : Qt::KeepAspectRatioByExpanding);
+          desiredSize.scale(width(), height(), mode);
           image = loadWallpaperFile(desiredSize); // load the scaled image
         }
       }
@@ -326,16 +330,7 @@ void DesktopWindow::updateWallpaper() {
       }
     }
     wallpaperPixmap_ = pixmap;
-    if(!pixmap.isNull()) {
-      QBrush brush(pixmap);
-      palette.setBrush(QPalette::Base, brush);
-    }
-    else // if the image is not loaded, fallback to use background color only
-      palette.setBrush(QPalette::Base, bgColor_);
   }
-
-  //FIXME: we should set the pixmap to X11 root window?
-  setPalette(palette);
 }
 
 void DesktopWindow::updateFromSettings(Settings& settings) {
@@ -343,8 +338,9 @@ void DesktopWindow::updateFromSettings(Settings& settings) {
   setWallpaperFile(settings.wallpaper());
   setWallpaperMode(settings.wallpaperMode());
   setFont(settings.desktopFont());
-  setIconSize(Fm::FolderView::IconMode, QSize(settings.bigIconSize(), settings.bigIconSize()));
-  // setIconSize may trigger relayout of items by QListView, so we need to do the layout again.
+  setIconSize(Fm::FolderView::IconMode, QSize(settings.desktopIconSize(), settings.desktopIconSize()));
+  setMargins(settings.desktopCellMargins());
+  // setIconSize and setMargins may trigger relayout of items by QListView, so we need to do the layout again.
   queueRelayout();
   setForeground(settings.desktopFgColor());
   setBackground(settings.desktopBgColor());
@@ -368,11 +364,11 @@ void DesktopWindow::prepareFileMenu(Fm::FileMenu* menu) {
   menu->insertSeparator(menu->separator2());
   menu->insertAction(menu->separator2(), action);
 
-  FmFileInfoList* files = menu->files();
+  Fm::FileInfoList files = menu->files();
   // select exactly one item
   if(fm_file_info_list_get_length(files) == 1) {
-    FmFileInfo* file = menu->firstFile();
-    if(customItemPos_.find(fm_file_info_get_name(file)) != customItemPos_.end()) {
+    Fm::FileInfo file = menu->firstFile();
+    if(customItemPos_.find(file.getName()) != customItemPos_.end()) {
       // the file item has a custom position
       action->setChecked(true);
     }
@@ -398,18 +394,24 @@ void DesktopWindow::onRowsInserted(const QModelIndex& parent, int start, int end
 }
 
 void DesktopWindow::onRowsAboutToBeRemoved(const QModelIndex& parent, int start, int end) {
+  Q_UNUSED(parent);
+  Q_UNUSED(start);
+  Q_UNUSED(end);
   if(!customItemPos_.isEmpty()) {
     // also delete stored custom item positions for the items currently being removed.
-    bool changed = false;
-    for(int row = start; row <= end ;++row) {
-      QModelIndex index = parent.child(row, 0);
-      FmFileInfo* file = proxyModel_->fileInfoFromIndex(index);
-      if(file) { // remove custom position for the item
-        if(customItemPos_.remove(fm_file_info_get_name(file)))
-          changed = true;
-      }
+    // Here we can't rely on ProxyFolderModel::fileInfoFromIndex() because, although rows
+    // aren't removed yet, files are already removed.
+    QHash<QByteArray, QPoint> _customItemPos = customItemPos_;
+    char* dektopPath = Fm::Path::getDesktop().toStr();
+    QString desktopDir = QString(dektopPath) + QString("/");
+    g_free(dektopPath);
+    QHash<QByteArray, QPoint>::iterator it;
+    for(it = _customItemPos.begin(); it != _customItemPos.end(); ++it) {
+      const QByteArray& name = it.key();
+      if(!QFile::exists(desktopDir + QString::fromUtf8(name, name.length())))
+        customItemPos_.remove(it.key());
     }
-    if(changed)
+    if(customItemPos_ != _customItemPos)
       saveItemPositions();
   }
   queueRelayout();
@@ -419,6 +421,40 @@ void DesktopWindow::onLayoutChanged() {
   queueRelayout();
 }
 
+void DesktopWindow::onModelSortFilterChanged() {
+  Settings& settings = static_cast<Application*>(qApp)->settings();
+  settings.setDesktopSortColumn(static_cast<Fm::FolderModel::ColumnId>(proxyModel_->sortColumn()));
+  settings.setDesktopSortOrder(proxyModel_->sortOrder());
+  settings.setSesktopSortFolderFirst(proxyModel_->folderFirst());
+}
+
+void DesktopWindow::onDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight) {
+  /****************************************************************************
+   NOTE: The display names of desktop entries and shortcuts may change without
+   their files being renamed and, on such occasions, a relayout will be needed.
+   Since there is no signal for that, we use the signal dataChanged() and the
+   QHash displayNames_, which remembers such display names with every relayout.
+   ****************************************************************************/
+  if(topLeft.column() == 0) {
+    bool relayout(false);
+    for(int i = topLeft.row(); i <= bottomRight.row(); ++i) {
+      QModelIndex index = topLeft.sibling(i, 0);
+      if(index.isValid() && displayNames_.contains(index)) {
+        Fm::FileInfo file = proxyModel_->fileInfoFromIndex(index);
+        if(displayNames_[index] != file.getDispName()) {
+          relayout = true;
+          break;
+        }
+      }
+    }
+    if(relayout) {
+      queueRelayout();
+      // parts of the old display name might still be visible if it's long
+      listView_->viewport()->update();
+    }
+  }
+}
+
 void DesktopWindow::onIndexesMoved(const QModelIndexList& indexes) {
   // remember the custom position for the items
   Q_FOREACH(const QModelIndex& index, indexes) {
@@ -428,20 +464,85 @@ void DesktopWindow::onIndexesMoved(const QModelIndexList& indexes) {
     // Since we only care about rows, not individual cells,
     // let's handle column 0 of every row here.
     if(index.column() == 0) {
-      FmFileInfo* file = proxyModel_->fileInfoFromIndex(index);
+      Fm::FileInfo file = proxyModel_->fileInfoFromIndex(index);
       QRect itemRect = listView_->rectForIndex(index);
-      QByteArray name = fm_file_info_get_name(file);
-      customItemPos_[name] = itemRect.topLeft();
-      // qDebug() << "indexMoved:" << name << index << itemRect;
+      QPoint tl = itemRect.topLeft();
+      QRect workArea = qApp->desktop()->availableGeometry(screenNum_);
+      workArea.adjust(12, 12, -12, -12);
+      if(customItemPos_.keys(tl).isEmpty() // don't put items on each other
+         && tl.x() >= workArea.x() && tl.y() >= workArea.y()
+         && tl.x() + listView_->gridSize().width() <= workArea.right() + 1 // for historical reasons (-> Qt doc)
+         && tl.y() + listView_->gridSize().height() <= workArea.bottom() + 1) { // as above
+        QByteArray name = file.getName();
+        customItemPos_[name] = tl;
+        // qDebug() << "indexMoved:" << name << index << itemRect;
+      }
     }
   }
   saveItemPositions();
   queueRelayout();
 }
 
+void DesktopWindow::removeBottomGap() {
+  /************************************************************
+   NOTE: Desktop is an area bounded from below while icons snap
+   to its grid srarting from above. Therefore, we try to adjust
+   the vertical cell margin to prevent relatively large gaps
+   from taking shape at the desktop bottom.
+   ************************************************************/
+  QSize cellMargins = getMargins();
+  int workAreaHeight = qApp->desktop()->availableGeometry(screenNum_).height()
+                       - 24; // a 12-pix margin will be considered everywhere
+  int cellHeight = listView_->gridSize().height() + listView_->spacing();
+  int iconNumber = workAreaHeight / cellHeight;
+  int bottomGap = workAreaHeight % cellHeight;
+  /*******************************************
+   First try to make room for an extra icon...
+   *******************************************/
+  // If one pixel is subtracted from the vertical margin, cellHeight
+  // will decrease by 2 while bottomGap will increase by 2*iconNumber.
+  // So, we can add an icon to the bottom once this inequality holds:
+  // bottomGap + 2*n*iconNumber >= cellHeight - 2*n
+  // From here, we get our "subtrahend":
+  qreal exactNumber = ((qreal)cellHeight - (qreal)bottomGap)
+                      / (2.0 * (qreal)iconNumber + 2.0);
+  int subtrahend = (int)exactNumber + ((int)exactNumber == exactNumber ? 0 : 1);
+  Settings& settings = static_cast<Application*>(qApp)->settings();
+  int minCellHeight = settings.desktopCellMargins().height();
+  if(subtrahend > 0
+     && cellMargins.height() - subtrahend >= minCellHeight) {
+    cellMargins -= QSize(0, subtrahend);
+  }
+  /***************************************************
+   ... but if that can't be done, try to spread icons!
+   ***************************************************/
+  else
+    cellMargins += QSize(0, (bottomGap / iconNumber) / 2);
+  // set the new margins (if they're changed)
+  delegate_->setMargins(cellMargins);
+  setMargins(cellMargins);
+  // in case the text shadow is reset to (0,0,0,0)
+  setShadow(settings.desktopShadowColor());
+}
+
+void DesktopWindow::paintBackground(QPaintEvent *event) {
+  // This is to workaround Qt bug 54384 which affects Qt >= 5.6
+  // https://bugreports.qt.io/browse/QTBUG-54384
+  // Since Qt does not paint the background of the QListView using the QPixmap we set properly, we do it ourselves.
+  QPainter painter(listView_->viewport()); // the painter paints on the viewport widget, not the QListView.
+  if(wallpaperMode_ == WallpaperNone || wallpaperPixmap_.isNull()) {
+    painter.fillRect(event->rect(), QBrush(bgColor_));
+  }
+  else {
+    painter.drawPixmap(event->rect(), wallpaperPixmap_, event->rect());
+  }
+}
+
 // QListView does item layout in a very inflexible way, so let's do our custom layout again.
 // FIXME: this is very inefficient, but due to the design flaw of QListView, this is currently the only workaround.
 void DesktopWindow::relayoutItems() {
+  displayNames_.clear();
+  loadItemPositions(); // something may have changed
   // qDebug("relayoutItems()");
   if(relayoutTimer_) {
     // this slot might be called from the timer, so we cannot delete it directly here.
@@ -468,12 +569,17 @@ void DesktopWindow::relayoutItems() {
     QPoint pos = workArea.topLeft();
     for(; row < rowCount; ++row) {
       QModelIndex index = proxyModel_->index(row, 0);
-      FmFileInfo* file = proxyModel_->fileInfoFromIndex(index);
-      QByteArray name = fm_file_info_get_name(file);
+      int itemWidth = delegate_->sizeHint(listView_->getViewOptions(), index).width();
+      Fm::FileInfo file = proxyModel_->fileInfoFromIndex(index);
+      // remember display names of desktop entries and shortcuts
+      if(file.isDesktopEntry() || file.isShortcut())
+        displayNames_[index] = QString(file.getDispName());
+      QByteArray name = file.getName();
       QHash<QByteArray, QPoint>::iterator it = customItemPos_.find(name);
       if(it != customItemPos_.end()) { // the item has a custom position
         QPoint customPos = *it;
-        listView_->setPositionForIndex(customPos, index);
+        // center the contents vertically
+        listView_->setPositionForIndex(customPos + QPoint((grid.width() - itemWidth) / 2, 0), index);
         // qDebug() << "set custom pos:" << name << row << index << customPos;
         continue;
       }
@@ -490,18 +596,19 @@ void DesktopWindow::relayoutItems() {
         --row;
       }
       else {
-        listView_->setPositionForIndex(pos, index);
+        // center the contents vertically
+        listView_->setPositionForIndex(pos + QPoint((grid.width() - itemWidth) / 2, 0), index);
         // qDebug() << "set pos" << name << row << index << pos;
       }
       // move to next cell in the column
       pos.setY(pos.y() + grid.height() + listView_->spacing());
-      if(pos.y() + grid.height() > workArea.bottom()) {
+      if(pos.y() + grid.height() > workArea.bottom() + 1) {
         // if the next position may exceed the bottom of work area, go to the top of next column
         pos.setX(pos.x() + grid.width() + listView_->spacing());
         pos.setY(workArea.top());
 
         // check if the new column exceeds the right margin of work area
-        if(pos.x() + grid.width() > workArea.right()) {
+        if(pos.x() + grid.width() > workArea.right() + 1) {
           if(desktop->isVirtualDesktop()) {
             // in virtual desktop mode, go to next screen
             ++screen;
@@ -517,14 +624,46 @@ void DesktopWindow::relayoutItems() {
 
 void DesktopWindow::loadItemPositions() {
   // load custom item positions
+  customItemPos_.clear();
   Settings& settings = static_cast<Application*>(qApp)->settings();
   QString configFile = QString("%1/desktop-items-%2.conf").arg(settings.profileDir(settings.profileName())).arg(screenNum_);
   QSettings file(configFile, QSettings::IniFormat);
+  QSize grid = listView_->gridSize();
+  QRect workArea = qApp->desktop()->availableGeometry(screenNum_);
+  workArea.adjust(12, 12, -12, -12);
+  char* dektopPath = Fm::Path::getDesktop().toStr();
+  QString desktopDir = QString(dektopPath) + QString("/");
+  g_free(dektopPath);
   Q_FOREACH(const QString& name, file.childGroups()) {
+    if(!QFile::exists(desktopDir + name.toUtf8())) {
+      // the file may have been removed from outside LXQT
+      continue;
+    }
     file.beginGroup(name);
     QVariant var = file.value("pos");
-    if(var.isValid())
-      customItemPos_[name.toUtf8()] = var.toPoint();
+    if(var.isValid()) {
+      QPoint customPos = var.toPoint();
+      if (customPos.x() >= workArea.x() && customPos.y() >= workArea.y()
+          && customPos.x() + listView_->gridSize().width() <= workArea.right() + 1
+          && customPos.y() + listView_->gridSize().height() <= workArea.bottom() + 1)
+      {
+        // correct positions that are't aligned to the grid
+        qreal w = qAbs((qreal)customPos.x() - (qreal)workArea.x())
+                  / (qreal)(grid.width() + listView_->spacing());
+        qreal h = qAbs(customPos.y() - (qreal)workArea.y())
+                  / (qreal)(grid.height() + listView_->spacing());
+        customPos.setX(workArea.x() + qRound(w) * (grid.width() + listView_->spacing()));
+        customPos.setY(workArea.y() + qRound(h) * (grid.height() + listView_->spacing()));
+        while(customItemPos_.values().contains(customPos)) {
+          customPos.setY(customPos.y() + grid.height() + listView_->spacing());
+          if(customPos.y() + grid.height() > workArea.bottom() + 1) {
+            customPos.setX(customPos.x() + grid.width() + listView_->spacing());
+            customPos.setY(workArea.top());
+          }
+        }
+        customItemPos_[name.toUtf8()] = customPos;
+      }
+    }
     file.endGroup();
   }
 }
@@ -554,8 +693,8 @@ void DesktopWindow::onStickToCurrentPos(bool toggled) {
 
   QModelIndexList indexes = listView_->selectionModel()->selectedIndexes();
   if(!indexes.isEmpty()) {
-    FmFileInfo* file = menu->firstFile();
-    QByteArray name = fm_file_info_get_name(file);
+    Fm::FileInfo file = menu->firstFile();
+    QByteArray name = file.getName();
     QModelIndex index = indexes.first();
     if(toggled) { // remember to current custom position
       QRect itemRect = listView_->rectForIndex(index);
@@ -575,6 +714,7 @@ void DesktopWindow::onStickToCurrentPos(bool toggled) {
 
 void DesktopWindow::queueRelayout(int delay) {
   // qDebug() << "queueRelayout";
+  removeBottomGap();
   if(!relayoutTimer_) {
     relayoutTimer_ = new QTimer();
     relayoutTimer_->setSingleShot(true);
@@ -586,16 +726,16 @@ void DesktopWindow::queueRelayout(int delay) {
 // slots for file operations
 
 void DesktopWindow::onCutActivated() {
-  if(FmPathList* paths = selectedFilePaths()) {
+  Fm::PathList paths = selectedFilePaths();
+  if(!paths.isNull()) {
     Fm::cutFilesToClipboard(paths);
-    fm_path_list_unref(paths);
   }
 }
 
 void DesktopWindow::onCopyActivated() {
-  if(FmPathList* paths = selectedFilePaths()) {
+  Fm::PathList paths = selectedFilePaths();
+  if(!paths.isNull()) {
     Fm::copyFilesToClipboard(paths);
-    fm_path_list_unref(paths);
   }
 }
 
@@ -604,31 +744,31 @@ void DesktopWindow::onPasteActivated() {
 }
 
 void DesktopWindow::onDeleteActivated() {
-  if(FmPathList* paths = selectedFilePaths()) {
+  Fm::PathList paths = selectedFilePaths();
+  if(!paths.isNull()) {
     Settings& settings = static_cast<Application*>(qApp)->settings();
     bool shiftPressed = (qApp->keyboardModifiers() & Qt::ShiftModifier ? true : false);
     if(settings.useTrash() && !shiftPressed)
       Fm::FileOperation::trashFiles(paths, settings.confirmTrash());
     else
       Fm::FileOperation::deleteFiles(paths, settings.confirmDelete());
-    fm_path_list_unref(paths);
   }
 }
 
 void DesktopWindow::onRenameActivated() {
-  if(FmFileInfoList* files = selectedFiles()) {
+  Fm::FileInfoList files = selectedFiles();
+  if(!files.isNull()) {
     for(GList* l = fm_file_info_list_peek_head_link(files); l; l = l->next) {
       FmFileInfo* info = FM_FILE_INFO(l->data);
       Fm::renameFile(info, NULL);
-      fm_file_info_list_unref(files);
     }
   }
 }
 
 void DesktopWindow::onFilePropertiesActivated() {
-  if(FmFileInfoList* files = selectedFiles()) {
+  Fm::FileInfoList files = selectedFiles();
+  if(!files.isNull()) {
     Fm::FilePropsDialog::showForFiles(files);
-    fm_file_info_list_unref(files);
   }
 }
 
@@ -743,6 +883,10 @@ bool DesktopWindow::eventFilter(QObject * watched, QEvent * event) {
   }
   else if(watched == listView_->viewport()) {
     switch(event->type()) {
+	case QEvent::Paint: {
+        paintBackground(static_cast<QPaintEvent*>(event));
+		break;
+	}
     case QEvent::MouseButtonPress:
     case QEvent::MouseButtonRelease:
         if(showWmMenu_) {
@@ -799,3 +943,5 @@ void DesktopWindow::setScreenNum(int num) {
     queueRelayout();
   }
 }
+
+} // namespace PCManFM
