@@ -27,6 +27,7 @@
 #include <QPoint>
 #include <QByteArray>
 #include <xcb/xcb.h>
+#include <libfm-qt/folder.h>
 
 namespace Fm {
   class CachedFolderModel;
@@ -49,7 +50,8 @@ public:
     WallpaperStretch,
     WallpaperFit,
     WallpaperCenter,
-    WallpaperTile
+    WallpaperTile,
+    WallpaperZoom
   };
 
   explicit DesktopWindow(int screenNum);
@@ -66,8 +68,6 @@ public:
   void updateWallpaper();
   void updateFromSettings(Settings& settings);
 
-  void xcbEvent(xcb_generic_event_t* generic_event);
-
   void queueRelayout(int delay = 0);
 
   int screenNum() const {
@@ -91,6 +91,7 @@ protected:
   virtual bool eventFilter(QObject * watched, QEvent * event);
 
   virtual void childDropEvent(QDropEvent* e);
+  virtual void closeEvent(QCloseEvent *event);
 
 protected Q_SLOTS:
   void onOpenDirRequested(FmPath* path, int target);
@@ -99,7 +100,9 @@ protected Q_SLOTS:
   void onRowsAboutToBeRemoved(const QModelIndex& parent, int start, int end);
   void onRowsInserted(const QModelIndex& parent, int start, int end);
   void onLayoutChanged();
+  void onModelSortFilterChanged();
   void onIndexesMoved(const QModelIndexList& indexes);
+  void onDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight);
 
   void relayoutItems();
   void onStickToCurrentPos(bool toggled);
@@ -114,10 +117,14 @@ protected Q_SLOTS:
   void onDeleteActivated();
   void onFilePropertiesActivated();
 
+private:
+  void removeBottomGap();
+  void paintBackground(QPaintEvent* event);
+
 private:
   Fm::ProxyFolderModel* proxyModel_;
   Fm::CachedFolderModel* model_;
-  FmFolder* folder_;
+  Fm::Folder folder_;
   Fm::FolderViewListView* listView_;
 
   QColor fgColor_;
@@ -132,6 +139,7 @@ private:
 
   int screenNum_;
   QHash<QByteArray, QPoint> customItemPos_;
+  QHash<QModelIndex, QString> displayNames_; // only for desktop entries and shortcuts
   QTimer* relayoutTimer_;
 };
 
