@@ -29,29 +29,39 @@
 #include <QToolButton>
 #include <QShortcut>
 #include <QKeySequence>
+#include <QDir>
+#include <QSettings>
 #include <QDebug>
 
 #include "tabpage.h"
-#include "filelauncher.h"
-#include "filemenu.h"
-#include "bookmarkaction.h"
-#include "fileoperation.h"
-#include "utilities.h"
-#include "filepropsdialog.h"
-#include "pathedit.h"
+#include "launcher.h"
+#include <libfm-qt/filemenu.h>
+#include <libfm-qt/bookmarkaction.h>
+#include <libfm-qt/fileoperation.h>
+#include <libfm-qt/utilities.h>
+#include <libfm-qt/filepropsdialog.h>
+#include <libfm-qt/pathedit.h>
+#include <libfm-qt/pathbar.h>
+#include <libfm-qt/path.h>
+#include <libfm-qt/fileinfo.h>
+#include <libfm-qt/folder.h>
 #include "ui_about.h"
 #include "application.h"
-#include "../libfm-qt/path.h"
-
-// #include "qmodeltest/modeltest.h"
 
 using namespace Fm;
 
 namespace PCManFM {
 
-MainWindow::MainWindow(FmPath* path):
+// static
+MainWindow* MainWindow::lastActive_ = nullptr;
+
+MainWindow::MainWindow(Path path):
   QMainWindow(),
-  fileLauncher_(this) {
+  pathEntry_(nullptr),
+  pathBar_(nullptr),
+  fileLauncher_(this),
+  rightClickIndex_(-1),
+  updatingViewMenu_(false) {
 
   Settings& settings = static_cast<Application*>(qApp)->settings();
   setAttribute(Qt::WA_DeleteOnClose);
@@ -84,6 +94,10 @@ MainWindow::MainWindow(FmPath* path):
   ui.tabBar->setElideMode(Qt::ElideRight);
   ui.tabBar->setExpanding(false);
   ui.tabBar->setMovable(true); // reorder the tabs by dragging
+  if(!settings.fullWidthTabBar()) {
+    ui.verticalLayout->removeWidget(ui.tabBar);
+    ui.verticalLayout_2->insertWidget(0, ui.tabBar);
+  }
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
   // switch to the tab under the cursor during dnd.
@@ -91,9 +105,15 @@ MainWindow::MainWindow(FmPath* path):
   ui.tabBar->setAcceptDrops(true);
 #endif
 
+  ui.tabBar->setContextMenuPolicy(Qt::CustomContextMenu);
+  connect(ui.actionCloseRight, &QAction::triggered, this, &MainWindow::closeRightTabs);
+  connect(ui.actionCloseLeft, &QAction::triggered, this, &MainWindow::closeLeftTabs);
+  connect(ui.actionCloseOther, &QAction::triggered, this, &MainWindow::closeOtherTabs);
+
   connect(ui.tabBar, &QTabBar::currentChanged, this, &MainWindow::onTabBarCurrentChanged);
   connect(ui.tabBar, &QTabBar::tabCloseRequested, this, &MainWindow::onTabBarCloseRequested);
   connect(ui.tabBar, &QTabBar::tabMoved, this, &MainWindow::onTabBarTabMoved);
+  connect(ui.tabBar, &QTabBar::customContextMenuRequested, this, &MainWindow::tabContextMenu);
   connect(ui.stackedWidget, &QStackedWidget::widgetRemoved, this, &MainWindow::onStackedWidgetWidgetRemoved);
 
   // FIXME: should we make the filter bar a per-view configuration?
@@ -115,13 +135,15 @@ MainWindow::MainWindow(FmPath* path):
   connect(ui.splitter, &QSplitter::splitterMoved, this, &MainWindow::onSplitterMoved);
 
   // path bar
-  pathEntry = new Fm::PathEdit(this);
-  connect(pathEntry, &Fm::PathEdit::returnPressed, this, &MainWindow::onPathEntryReturnPressed);
-  ui.toolBar->insertWidget(ui.actionGo, pathEntry);
+  createPathBar(settings.pathBarButtons());
+  if(settings.pathBarButtons())
+    ui.actionPathButtons->setChecked(true);
+  else
+    ui.actionLocationBar->setChecked(true);
 
   // add filesystem info to status bar
-  fsInfoLabel = new QLabel(ui.statusbar);
-  ui.statusbar->addPermanentWidget(fsInfoLabel);
+  fsInfoLabel_ = new QLabel(ui.statusbar);
+  ui.statusbar->addPermanentWidget(fsInfoLabel_);
 
   // setup the splitter
   ui.splitter->setStretchFactor(1, 1); // only the right pane can be stretched
@@ -131,8 +153,8 @@ MainWindow::MainWindow(FmPath* path):
   ui.splitter->setSizes(sizes);
 
   // load bookmark menu
-  bookmarks = fm_bookmarks_dup();
-  g_signal_connect(bookmarks, "changed", G_CALLBACK(onBookmarksChanged), this);
+  bookmarks_ = Fm::Bookmarks::dup();
+  g_signal_connect(bookmarks_, "changed", G_CALLBACK(onBookmarksChanged), this);
   loadBookmarksMenu();
 
   // Fix the menu groups which is not done by Qt designer
@@ -157,13 +179,47 @@ MainWindow::MainWindow(FmPath* path):
   group->addAction(ui.actionAscending);
   group->addAction(ui.actionDescending);
 
+  group = new QActionGroup(ui.menuPathBarStyle);
+  group->setExclusive(true);
+  group->addAction(ui.actionLocationBar);
+  group->addAction(ui.actionPathButtons);
+
+  // Add menubar actions to the main window this is necessary so that actions
+  // shortcuts are still working when the menubar is hidden.
+  addActions(ui.menubar->actions());
+
+  // Show or hide the menu bar
+  QMenu *menu = new QMenu(ui.toolBar);
+  menu->addMenu(ui.menu_File);
+  menu->addMenu(ui.menu_Editw);
+  menu->addMenu(ui.menu_View);
+  menu->addMenu(ui.menu_Go);
+  menu->addMenu(ui.menu_Bookmarks);
+  menu->addMenu(ui.menu_Tool);
+  menu->addMenu(ui.menu_Help);
+  ui.actionMenu->setMenu(menu);
+  if(ui.actionMenu->icon().isNull())
+    ui.actionMenu->setIcon(QIcon::fromTheme("applications-system"));
+  QToolButton* menuBtn = static_cast<QToolButton*>(ui.toolBar->widgetForAction(ui.actionMenu));
+  menuBtn->setPopupMode(QToolButton::InstantPopup);
+
+  menuSep_ = ui.toolBar->insertSeparator(ui.actionMenu);
+  menuSep_->setVisible(!settings.showMenuBar());
+  ui.actionMenu->setVisible(!settings.showMenuBar());
+  ui.menubar->setVisible(settings.showMenuBar());
+  ui.actionMenu_bar->setChecked(settings.showMenuBar());
+  connect(ui.actionMenu_bar, &QAction::triggered, this, &MainWindow::toggleMenuBar);
+
   // create shortcuts
   QShortcut* shortcut;
+  shortcut = new QShortcut(QKeySequence(Qt::Key_Escape), this);
+  connect(shortcut, &QShortcut::activated, this, &MainWindow::onResetFocus);
+
   shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_L), this);
-  connect(shortcut, &QShortcut::activated, pathEntry, static_cast<void (QWidget::*)()>(&Fm::PathEdit::setFocus));
+  connect(shortcut, &QShortcut::activated, this, &MainWindow::focusPathEntry);
 
   shortcut = new QShortcut(Qt::ALT + Qt::Key_D, this);
-  connect(shortcut, &QShortcut::activated, pathEntry, static_cast<void (QWidget::*)()>(&QWidget::setFocus));
+  connect(shortcut, &QShortcut::activated, this, &MainWindow::focusPathEntry);
 
   shortcut = new QShortcut(Qt::CTRL + Qt::Key_Tab, this);
   connect(shortcut, &QShortcut::activated, this, &MainWindow::onShortcutNextTab);
@@ -171,6 +227,15 @@ MainWindow::MainWindow(FmPath* path):
   shortcut = new QShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_Tab, this);
   connect(shortcut, &QShortcut::activated, this, &MainWindow::onShortcutPrevTab);
 
+  // Add Ctrl+PgUp and Ctrl+PgDown as well, because they are common in Firefox
+  // , Opera, Google Chromium/Google Chrome and most other tab-using
+  // applications.
+  shortcut = new QShortcut(Qt::CTRL + Qt::Key_PageDown, this);
+  connect(shortcut, &QShortcut::activated, this, &MainWindow::onShortcutNextTab);
+
+  shortcut = new QShortcut(Qt::CTRL + Qt::Key_PageUp, this);
+  connect(shortcut, &QShortcut::activated, this, &MainWindow::onShortcutPrevTab);
+
   int i;
   for(i = 0; i < 10; ++i) {
     shortcut = new QShortcut(QKeySequence(Qt::ALT + Qt::Key_0 + i), this);
@@ -186,13 +251,20 @@ MainWindow::MainWindow(FmPath* path):
   shortcut = new QShortcut(QKeySequence(Qt::SHIFT + Qt::Key_Delete), this);
   connect(shortcut, &QShortcut::activated, this, &MainWindow::on_actionDelete_triggered);
 
+  shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_I), this);
+  connect(shortcut, &QShortcut::activated, this, &MainWindow::focusFilterBar);
+
+  // in addition to F3, for convenience
+  shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_F), this);
+  connect(shortcut, &QShortcut::activated, ui.actionFindFiles, &QAction::trigger);
+
   if(QToolButton* clearButton = ui.filterBar->findChild<QToolButton*>()) {
     clearButton->setToolTip(tr("Clear text (Ctrl+K)"));
     shortcut = new QShortcut(Qt::CTRL + Qt::Key_K, this);
     connect(shortcut, &QShortcut::activated, ui.filterBar, &QLineEdit::clear);
   }
 
-  if(path)
+  if(!path.isNull())
     addTab(path);
 
   // size from settings
@@ -201,14 +273,18 @@ MainWindow::MainWindow(FmPath* path):
     if(settings.windowMaximized())
       setWindowState(windowState() | Qt::WindowMaximized);
   }
+
+  if(QApplication::layoutDirection() == Qt::RightToLeft)
+    setRTLIcons(true);
 }
 
 MainWindow::~MainWindow() {
-  if(bookmarks)
-    g_object_unref(bookmarks);
+  if(!bookmarks_.isNull()) {
+    g_signal_handlers_disconnect_by_func(bookmarks_, (gpointer)G_CALLBACK(onBookmarksChanged), this);
+  }
 }
 
-void MainWindow::chdir(FmPath* path) {
+void MainWindow::chdir(Path path) {
   TabPage* page = currentPage();
 
   if(page) {
@@ -218,8 +294,25 @@ void MainWindow::chdir(FmPath* path) {
   }
 }
 
+void MainWindow::createPathBar(bool usePathButtons) {
+  QWidget* bar;
+  if(usePathButtons) {
+    bar = pathBar_ = new Fm::PathBar(this);
+    connect(pathBar_, &Fm::PathBar::chdir, this, &MainWindow::onPathBarChdir);
+    connect(pathBar_, &Fm::PathBar::middleClickChdir, this, &MainWindow::onPathBarMiddleClickChdir);
+    connect(pathBar_, &Fm::PathBar::editingFinished, this, &MainWindow::onResetFocus);
+  }
+  else {
+    bar = pathEntry_ = new Fm::PathEdit(this);
+    connect(pathEntry_, &Fm::PathEdit::returnPressed, this, &MainWindow::onPathEntryReturnPressed);
+    connect(pathEntry_, &QLineEdit::textEdited, this, &MainWindow::onPathEntryEdited);
+  }
+  ui.toolBar->insertWidget(ui.actionGo, bar);
+  ui.actionGo->setVisible(!usePathButtons);
+}
+
 // add a new tab
-void MainWindow::addTab(FmPath* path) {
+int MainWindow::addTab(Path path) {
   Settings& settings = static_cast<Application*>(qApp)->settings();
 
   TabPage* newPage = new TabPage(path, this);
@@ -237,14 +330,55 @@ void MainWindow::addTab(FmPath* path) {
   if(!settings.alwaysShowTabs()) {
     ui.tabBar->setVisible(ui.tabBar->count() > 1);
   }
+
+  return index;
+}
+
+void MainWindow::toggleMenuBar(bool checked) {
+  Settings& settings = static_cast<Application*>(qApp)->settings();
+  bool showMenuBar = !settings.showMenuBar();
+
+  if (!showMenuBar) {
+    if (QMessageBox::Cancel == QMessageBox::warning(this,
+          tr("Hide menu bar"),
+          tr("This will hide the menu bar completely, use Ctrl+M to show it again."),
+          QMessageBox::Ok | QMessageBox::Cancel)) {
+      ui.actionMenu_bar->setChecked(true);
+      return;
+    }
+  }
+
+  ui.menubar->setVisible(showMenuBar);
+  ui.actionMenu_bar->setChecked(showMenuBar);
+  menuSep_->setVisible(!showMenuBar);
+  ui.actionMenu->setVisible(!showMenuBar);
+  settings.setShowMenuBar(showMenuBar);
 }
 
 void MainWindow::onPathEntryReturnPressed() {
-  QString text = pathEntry->text();
+  QString text = pathEntry_->text();
   QByteArray utext = text.toUtf8();
-  FmPath* path = fm_path_new_for_display_name(utext);
-  chdir(path);
-  fm_path_unref(path);
+  chdir(Fm::Path::newForDisplayName(utext));
+}
+
+void MainWindow::onPathEntryEdited(const QString& text) {
+  QString realText(text);
+  if(realText == "~" || realText.startsWith("~/")) {
+    realText.replace(0, 1, QDir::homePath());
+    pathEntry_->setText(realText);
+  }
+}
+
+void MainWindow::onPathBarChdir(FmPath* dirPath) {
+  // call chdir() only when needed because otherwise
+  // filter bar will be cleard on changing current tab
+  TabPage* page = currentPage();
+  if(page && dirPath != page->path())
+    chdir(dirPath);
+}
+
+void MainWindow::onPathBarMiddleClickChdir(FmPath* dirPath) {
+  addTab(dirPath);
 }
 
 void MainWindow::on_actionGoUp_triggered() {
@@ -278,11 +412,18 @@ void MainWindow::on_actionGoForward_triggered() {
 }
 
 void MainWindow::on_actionHome_triggered() {
-  chdir(fm_path_get_home());
+  chdir(Fm::Path::getHome());
 }
 
 void MainWindow::on_actionReload_triggered() {
   currentPage()->reload();
+  if(pathEntry_ != nullptr)
+    pathEntry_->setText(currentPage()->pathName());
+}
+
+void MainWindow::on_actionConnectToServer_triggered() {
+  Application* app = static_cast<Application*>(qApp);
+  app->connectToServer();
 }
 
 void MainWindow::on_actionGo_triggered() {
@@ -290,18 +431,19 @@ void MainWindow::on_actionGo_triggered() {
 }
 
 void MainWindow::on_actionNewTab_triggered() {
-  FmPath* path = currentPage()->path();
-  addTab(path);
+  Fm::Path path = currentPage()->path();
+  int index = addTab(path);
+  ui.tabBar->setCurrentIndex(index);
 }
 
 void MainWindow::on_actionNewWin_triggered() {
-  FmPath* path = currentPage()->path();
+  Fm::Path path = currentPage()->path();
   (new MainWindow(path))->show();
 }
 
 void MainWindow::on_actionNewFolder_triggered() {
   if(TabPage* tabPage = currentPage()) {
-    FmPath* dirPath = tabPage->folderView()->path();
+    Fm::Path dirPath = tabPage->folderView()->path();
 
     if(dirPath)
       createFileOrFolder(CreateNewFolder, dirPath);
@@ -310,7 +452,7 @@ void MainWindow::on_actionNewFolder_triggered() {
 
 void MainWindow::on_actionNewBlankFile_triggered() {
   if(TabPage* tabPage = currentPage()) {
-    FmPath* dirPath = tabPage->folderView()->path();
+    Fm::Path dirPath = tabPage->folderView()->path();
 
     if(dirPath)
       createFileOrFolder(CreateNewTextFile, dirPath);
@@ -329,13 +471,10 @@ void MainWindow::on_actionCloseWindow_triggered() {
 
 void MainWindow::on_actionFileProperties_triggered() {
   TabPage* page = currentPage();
-
   if(page) {
-    FmFileInfoList* files = page->selectedFiles();
-
-    if(files) {
+    Fm::FileInfoList files = page->selectedFiles();
+    if(!files.isNull()) {
       Fm::FilePropsDialog::showForFiles(files);
-      fm_file_info_list_unref(files);
     }
   }
 }
@@ -344,22 +483,18 @@ void MainWindow::on_actionFolderProperties_triggered() {
   TabPage* page = currentPage();
 
   if(page) {
-    FmFolder* folder = page->folder();
-
-    if(folder) {
-      FmFileInfo* info = fm_folder_get_info(folder);
-
-      if(info)
+    Fm::Folder folder = page->folder();
+    if(!folder.isNull()) {
+      Fm::FileInfo info = folder.getInfo();
+      if(!info.isNull())
         Fm::FilePropsDialog::showForFile(info);
     }
   }
 }
 
 void MainWindow::on_actionShowHidden_triggered(bool checked) {
-  TabPage* tabPage = currentPage();
-  tabPage->setShowHidden(checked);
+  currentPage()->setShowHidden(checked);
   ui.sidePane->setShowHidden(checked);
-  static_cast<Application*>(qApp)->settings().setShowHidden(checked);
 }
 
 void MainWindow::on_actionByFileName_triggered(bool checked) {
@@ -398,44 +533,84 @@ void MainWindow::on_actionFolderFirst_triggered(bool checked) {
   currentPage()->setSortFolderFirst(checked);
 }
 
+void MainWindow::on_actionPreserveView_triggered(bool checked) {
+  TabPage* page = currentPage();
+  page->setCustomizedView(!page->hasCustomizedView());
+}
+
 void MainWindow::on_actionFilter_triggered(bool checked) {
   ui.filterBar->setVisible(checked);
+  if(checked)
+    ui.filterBar->setFocus();
+  else if(TabPage* tabPage = currentPage()) {
+    ui.filterBar->clear();
+    tabPage->folderView()->childView()->setFocus();
+    // clear filter string for all tabs
+    int n = ui.stackedWidget->count();
+    for(int i = 0; i < n; ++i) {
+      TabPage* page = static_cast<TabPage*>(ui.stackedWidget->widget(i));
+      if(!page->getFilterStr().isEmpty()) {
+        page->setFilterStr(QString());
+        page->applyFilter();
+      }
+    }
+  }
   static_cast<Application*>(qApp)->settings().setShowFilter(checked);
 }
 
+void MainWindow::on_actionLocationBar_triggered(bool checked) {
+  if(checked) {
+    // show current path in a location bar entry
+    if(pathBar_ != nullptr) {
+      delete pathBar_;
+      pathBar_ = nullptr;
+    }
+    createPathBar(false);
+    pathEntry_->setText(currentPage()->pathName());
+    static_cast<Application*>(qApp)->settings().setPathBarButtons(false);
+  }
+}
+
+void MainWindow::on_actionPathButtons_triggered(bool checked) {
+  if(checked && pathBar_ == nullptr) {
+    // show current path as buttons
+    if(pathEntry_ != nullptr) {
+      delete pathEntry_;
+      pathEntry_ = nullptr;
+    }
+    createPathBar(true);
+    pathBar_->setPath(currentPage()->path());
+    static_cast<Application*>(qApp)->settings().setPathBarButtons(true);
+  }
+}
+
 void MainWindow::on_actionComputer_triggered() {
-  FmPath* path = fm_path_new_for_uri("computer:///");
-  chdir(path);
-  fm_path_unref(path);
+  chdir(Fm::Path::newForUri("computer:///"));
 }
 
 void MainWindow::on_actionApplications_triggered() {
-  chdir(fm_path_get_apps_menu());
+  chdir(Fm::Path::getAppsMenu());
 }
 
 void MainWindow::on_actionTrash_triggered() {
-  chdir(fm_path_get_trash());
+  chdir(Fm::Path::getTrash());
 }
 
 void MainWindow::on_actionNetwork_triggered() {
-  FmPath* path = fm_path_new_for_uri("network:///");
-  chdir(path);
-  fm_path_unref(path);
+  chdir(Fm::Path::newForUri("network:///"));
 }
 
 void MainWindow::on_actionDesktop_triggered() {
-  chdir(fm_path_get_desktop());
+  chdir(Fm::Path::getDesktop());
 }
 
 void MainWindow::on_actionAddToBookmarks_triggered() {
   TabPage* page = currentPage();
-
   if(page) {
-    FmPath* cwd = page->path();
-
-    if(cwd) {
-      char* dispName = fm_path_display_basename(cwd);
-      fm_bookmarks_insert(bookmarks, cwd, dispName, -1);
+    Fm::Path cwd = page->path();
+    if(!cwd.isNull()) {
+      char* dispName = cwd.displayBasename();
+      bookmarks_.insert(cwd, dispName, -1);
       g_free(dispName);
     }
   }
@@ -481,6 +656,12 @@ void MainWindow::onTabBarCloseRequested(int index) {
   closeTab(index);
 }
 
+void MainWindow::onResetFocus() {
+  if(TabPage* page = currentPage()) {
+    page->folderView()->childView()->setFocus();
+  }
+}
+
 void MainWindow::onTabBarTabMoved(int from, int to) {
   // a tab in the tab bar is moved by the user, so we have to move the
   //  corredponding tab page in the stacked widget to the new position, too.
@@ -497,6 +678,13 @@ void MainWindow::onTabBarTabMoved(int from, int to) {
   }
 }
 
+void MainWindow::focusFilterBar() {
+  if(!ui.filterBar->isVisible())
+    ui.actionFilter->trigger();
+  else
+    ui.filterBar->setFocus();
+}
+
 void MainWindow::onFilterStringChanged(QString str) {
   if(TabPage* tabPage = currentPage()) {
     // appy filter only if needed (not if tab is changed)
@@ -532,6 +720,9 @@ void MainWindow::resizeEvent(QResizeEvent *event) {
 
 void MainWindow::closeEvent(QCloseEvent *event)
 {
+  if(lastActive_ == this)
+    lastActive_ = nullptr;
+
   QWidget::closeEvent(event);
   Settings& settings = static_cast<Application*>(qApp)->settings();
   if(settings.rememberWindowSize()) {
@@ -559,16 +750,19 @@ void MainWindow::updateStatusBarForCurrentPage() {
   ui.statusbar->showMessage(text);
 
   text = tabPage->statusText(TabPage::StatusTextFSInfo);
-  fsInfoLabel->setText(text);
-  fsInfoLabel->setVisible(!text.isEmpty());
+  fsInfoLabel_->setText(text);
+  fsInfoLabel_->setVisible(!text.isEmpty());
 }
 
 void MainWindow::updateViewMenuForCurrentPage() {
+  if(updatingViewMenu_)  // prevent recursive calls
+    return;
+  updatingViewMenu_ = true;
   TabPage* tabPage = currentPage();
-
   if(tabPage) {
     // update menus. FIXME: should we move this to another method?
     ui.actionShowHidden->setChecked(tabPage->showHidden());
+    ui.actionPreserveView->setChecked(tabPage->hasCustomizedView());
 
     // view mode
     QAction* modeAction = NULL;
@@ -607,10 +801,10 @@ void MainWindow::updateViewMenuForCurrentPage() {
       ui.actionAscending->setChecked(true);
     else
       ui.actionDescending->setChecked(true);
-
     ui.actionCaseSensitive->setChecked(tabPage->sortCaseSensitive());
     ui.actionFolderFirst->setChecked(tabPage->sortFolderFirst());
   }
+  updatingViewMenu_ = false;
 }
 
 void MainWindow::updateUIForCurrentPage() {
@@ -618,9 +812,14 @@ void MainWindow::updateUIForCurrentPage() {
 
   if(tabPage) {
     setWindowTitle(tabPage->title());
-    pathEntry->setText(tabPage->pathName());
+    if(pathEntry_ != nullptr) {
+      pathEntry_->setText(tabPage->pathName());
+    }
+    else if(pathBar_ != nullptr) {
+      pathBar_->setPath(tabPage->path());
+    }
     ui.statusbar->showMessage(tabPage->statusText());
-    fsInfoLabel->setText(tabPage->statusText(TabPage::StatusTextFSInfo));
+    fsInfoLabel_->setText(tabPage->statusText(TabPage::StatusTextFSInfo));
     tabPage->folderView()->childView()->setFocus();
 
     // update side pane
@@ -679,8 +878,8 @@ void MainWindow::onTabPageStatusChanged(int type, QString statusText) {
       break;
     }
     case TabPage::StatusTextFSInfo:
-      fsInfoLabel->setText(tabPage->statusText(TabPage::StatusTextFSInfo));
-      fsInfoLabel->setVisible(!statusText.isEmpty());
+      fsInfoLabel_->setText(tabPage->statusText(TabPage::StatusTextFSInfo));
+      fsInfoLabel_->setVisible(!statusText.isEmpty());
       break;
     }
   }
@@ -696,22 +895,23 @@ void MainWindow::onTabPageOpenDirRequested(FmPath* path, int target) {
     addTab(path);
     break;
 
-  case OpenInNewWindow: {
+  case OpenInNewWindow:
     (new MainWindow(path))->show();
     break;
   }
-  }
 }
 
 void MainWindow::onTabPageSortFilterChanged() {
   TabPage* tabPage = static_cast<TabPage*>(sender());
-
   if(tabPage == currentPage()) {
     updateViewMenuForCurrentPage();
-    Settings& settings = static_cast<Application*>(qApp)->settings();
-    settings.setSortColumn(static_cast<Fm::FolderModel::ColumnId>(tabPage->sortColumn()));
-    settings.setSortOrder(tabPage->sortOrder());
-    settings.setSortFolderFirst(tabPage->sortFolderFirst());
+    if(!tabPage->hasCustomizedView()) {
+      Settings& settings = static_cast<Application*>(qApp)->settings();
+      settings.setSortColumn(static_cast<Fm::FolderModel::ColumnId>(tabPage->sortColumn()));
+      settings.setSortOrder(tabPage->sortOrder());
+      settings.setSortFolderFirst(tabPage->sortFolderFirst());
+      settings.setSortCaseSensitive(tabPage->sortCaseSensitive());
+    }
   }
 }
 
@@ -753,7 +953,7 @@ void MainWindow::onSplitterMoved(int pos, int index) {
 }
 
 void MainWindow::loadBookmarksMenu() {
-  GList* allBookmarks = fm_bookmarks_get_all(bookmarks);
+  GList* allBookmarks = bookmarks_.getAll();
   QAction* before = ui.actionAddToBookmarks;
 
   for(GList* l = allBookmarks; l; l = l->next) {
@@ -784,7 +984,7 @@ void MainWindow::onBookmarksChanged(FmBookmarks* bookmarks, MainWindow* pThis) {
 
 void MainWindow::onBookmarkActionTriggered() {
   BookmarkAction* action = static_cast<BookmarkAction*>(sender());
-  FmPath* path = action->path();
+  Fm::Path path = action->path();
   if(path) {
     Application* app = static_cast<Application*>(qApp);
     Settings& settings = app->settings();
@@ -805,16 +1005,14 @@ void MainWindow::onBookmarkActionTriggered() {
 
 void MainWindow::on_actionCopy_triggered() {
   TabPage* page = currentPage();
-  FmPathList* paths = page->selectedFilePaths();
+  Fm::PathList paths = page->selectedFilePaths();
   copyFilesToClipboard(paths);
-  fm_path_list_unref(paths);
 }
 
 void MainWindow::on_actionCut_triggered() {
   TabPage* page = currentPage();
-  FmPathList* paths = page->selectedFilePaths();
+  Fm::PathList paths = page->selectedFilePaths();
   cutFilesToClipboard(paths);
-  fm_path_list_unref(paths);
 }
 
 void MainWindow::on_actionPaste_triggered() {
@@ -825,26 +1023,22 @@ void MainWindow::on_actionDelete_triggered() {
   Application* app = static_cast<Application*>(qApp);
   Settings& settings = app->settings();
   TabPage* page = currentPage();
-  FmPathList* paths = page->selectedFilePaths();
+  Fm::PathList paths = page->selectedFilePaths();
 
   bool shiftPressed = (qApp->keyboardModifiers() & Qt::ShiftModifier ? true : false);
   if(settings.useTrash() && !shiftPressed)
     FileOperation::trashFiles(paths, settings.confirmTrash(), this);
   else
     FileOperation::deleteFiles(paths, settings.confirmDelete(), this);
-
-  fm_path_list_unref(paths);
 }
 
 void MainWindow::on_actionRename_triggered() {
   TabPage* page = currentPage();
-  FmFileInfoList* files = page->selectedFiles();
-
+  Fm::FileInfoList files = page->selectedFiles();
   for(GList* l = fm_file_info_list_peek_head_link(files); l; l = l->next) {
     FmFileInfo* file = FM_FILE_INFO(l->data);
     Fm::renameFile(file, NULL);
   }
-  fm_file_info_list_unref(files);
 }
 
 
@@ -861,15 +1055,44 @@ void MainWindow::on_actionPreferences_triggered() {
   app->preferences(QString());
 }
 
-/*
-void MainWindow::changeEvent(QEvent* event) {
+// change some icons according to layout direction
+void MainWindow::setRTLIcons(bool isRTL) {
+  QIcon nxtIcn = QIcon::fromTheme("go-next");
+  QIcon prevIcn = QIcon::fromTheme("go-previous");
+  if(isRTL) {
+    ui.actionGoBack->setIcon(nxtIcn);
+    ui.actionCloseLeft->setIcon(nxtIcn);
+    ui.actionGoForward->setIcon(prevIcn);
+    ui.actionCloseRight->setIcon(prevIcn);
+  }
+  else {
+    ui.actionGoBack->setIcon(prevIcn);
+    ui.actionCloseLeft->setIcon(prevIcn);
+    ui.actionGoForward->setIcon(nxtIcn);
+    ui.actionCloseRight->setIcon(nxtIcn);
+  }
+}
+
+bool MainWindow::event(QEvent* event) {
   switch(event->type()) {
-    case QEvent::StyleChange:
+  case QEvent::WindowActivate:
+    lastActive_ = this;
+  default:
+    break;
+  }
+  return QMainWindow::event(event);
+}
+
+void MainWindow::changeEvent(QEvent *event) {
+  switch(event->type()) {
+    case QEvent::LayoutDirectionChange:
+      setRTLIcons(QApplication::layoutDirection() == Qt::RightToLeft);
+      break;
+    default:
       break;
   }
   QWidget::changeEvent(event);
 }
-*/
 
 void MainWindow::onBackForwardContextMenu(QPoint pos) {
   // show a popup menu for browsing history here.
@@ -881,7 +1104,9 @@ void MainWindow::onBackForwardContextMenu(QPoint pos) {
   for(int i = 0; i < history.size(); ++i) {
     const BrowseHistoryItem& item = history.at(i);
     Fm::Path path = item.path();
-    QAction* action = menu.addAction(path.displayName());
+    char* name = path.displayName(true);
+    QAction* action = menu.addAction(QString::fromUtf8(name));
+    g_free(name);
     if(i == current) {
       // make the current path bold and checked
       action->setCheckable(true);
@@ -900,6 +1125,50 @@ void MainWindow::onBackForwardContextMenu(QPoint pos) {
   }
 }
 
+void MainWindow::tabContextMenu(const QPoint& pos) {
+  int tabNum = ui.tabBar->count();
+  if(tabNum <= 1) return;
+
+  rightClickIndex_ = ui.tabBar->tabAt(pos);
+  if(rightClickIndex_ < 0) return;
+
+  QMenu menu;
+  if(rightClickIndex_ > 0)
+      menu.addAction(ui.actionCloseLeft);
+  if(rightClickIndex_ < tabNum - 1) {
+    menu.addAction(ui.actionCloseRight);
+    if(rightClickIndex_ > 0) {
+      menu.addSeparator();
+      menu.addAction(ui.actionCloseOther);
+    }
+  }
+  menu.exec(ui.tabBar->mapToGlobal(pos));
+}
+
+void MainWindow::closeLeftTabs() {
+  while(rightClickIndex_ > 0) {
+    closeTab(rightClickIndex_ - 1);
+    --rightClickIndex_;
+  }
+}
+
+void MainWindow::closeRightTabs() {
+  if(rightClickIndex_ < 0) return;
+  while(rightClickIndex_ < ui.tabBar->count() - 1)
+    closeTab(rightClickIndex_ + 1);
+}
+
+void MainWindow::focusPathEntry() {
+  // use text entry for the path bar
+  if(pathEntry_ != nullptr) {
+    pathEntry_->setFocus();
+    pathEntry_->selectAll();
+  }
+  else if (pathBar_ != nullptr) { // use button-style path bar
+    pathBar_->openEditor();
+  }
+}
+
 void MainWindow::updateFromSettings(Settings& settings) {
   // apply settings
 
@@ -912,9 +1181,16 @@ void MainWindow::updateFromSettings(Settings& settings) {
 
   // tabs
   ui.tabBar->setTabsClosable(settings.showTabClose());
-
-  if(!settings.alwaysShowTabs()) {
-    ui.tabBar->setVisible(ui.tabBar->count() > 1);
+  ui.tabBar->setVisible(settings.alwaysShowTabs() || (ui.tabBar->count() > 1));
+  if(ui.verticalLayout->indexOf(ui.tabBar) > -1) {
+    if(!settings.fullWidthTabBar()) {
+      ui.verticalLayout->removeWidget(ui.tabBar);
+      ui.verticalLayout_2->insertWidget(0, ui.tabBar);
+    }
+  }
+  else if (ui.verticalLayout_2->indexOf(ui.tabBar) > -1 && settings.fullWidthTabBar()) {
+    ui.verticalLayout_2->removeWidget(ui.tabBar);
+    ui.verticalLayout->insertWidget(0, ui.tabBar);
   }
 
   // all tab pages
@@ -960,9 +1236,9 @@ void MainWindow::on_actionOpenAsRoot_triggered() {
       g_free(cmd);
 
       if(appInfo) {
-        FmPath* cwd = page->path();
+        Fm::Path cwd = page->path();
         GError* err = NULL;
-        char* uri = fm_path_to_uri(cwd);
+        char* uri = cwd.toUri();
         GList* uris = g_list_prepend(NULL, uri);
 
         if(!g_app_info_launch_uris(appInfo, uris, NULL, &err)) {
@@ -985,16 +1261,15 @@ void MainWindow::on_actionOpenAsRoot_triggered() {
 
 void MainWindow::on_actionFindFiles_triggered() {
   Application* app = static_cast<Application*>(qApp);
-  FmPathList* selectedPaths = currentPage()->selectedFilePaths();
+  Fm::PathList selectedPaths = currentPage()->selectedFilePaths();
   QStringList paths;
-  if(selectedPaths) {
+  if(!selectedPaths.isNull()) {
     for(GList* l = fm_path_list_peek_head_link(selectedPaths); l; l = l->next) {
       // FIXME: is it ok to use display name here?
       // This might be broken on filesystems with non-UTF-8 filenames.
       Fm::Path path(FM_PATH(l->data));
       paths.append(path.displayName(false));
     }
-    fm_path_list_unref(selectedPaths);
   }
   else {
     paths.append(currentPage()->pathName());
