@@ -32,6 +32,13 @@
 #include <QStackedWidget>
 #include <QSplitter>
 #include "launcher.h"
+#include <libfm-qt/bookmarks.h>
+#include <libfm-qt/path.h>
+
+namespace Fm {
+  class PathEdit;
+  class PathBar;
+}
 
 namespace PCManFM {
 
@@ -41,11 +48,11 @@ class Settings;
 class MainWindow : public QMainWindow {
 Q_OBJECT
 public:
-  MainWindow(FmPath* path = NULL);
+  MainWindow(Fm::Path path = Fm::Path());
   virtual ~MainWindow();
 
-  void chdir(FmPath* path);
-  void addTab(FmPath* path);
+  void chdir(Fm::Path path);
+  int addTab(Fm::Path path);
 
   TabPage* currentPage() {
     return reinterpret_cast<TabPage*>(ui.stackedWidget->currentWidget());
@@ -53,9 +60,16 @@ public:
 
   void updateFromSettings(Settings& settings);
 
+  static MainWindow* lastActive() {
+    return lastActive_;
+  }
+
 protected Q_SLOTS:
 
   void onPathEntryReturnPressed();
+  void onPathEntryEdited(const QString& text);
+  void onPathBarChdir(FmPath* dirPath);
+  void onPathBarMiddleClickChdir(FmPath* dirPath);
 
   void on_actionNewTab_triggered();
   void on_actionNewWin_triggered();
@@ -80,6 +94,7 @@ protected Q_SLOTS:
   void on_actionGoUp_triggered();
   void on_actionHome_triggered();
   void on_actionReload_triggered();
+  void on_actionConnectToServer_triggered();
 
   void on_actionIconView_triggered();
   void on_actionCompactView_triggered();
@@ -88,6 +103,7 @@ protected Q_SLOTS:
 
   void on_actionGo_triggered();
   void on_actionShowHidden_triggered(bool check);
+  void on_actionPreserveView_triggered(bool checked);
 
   void on_actionByFileName_triggered(bool checked);
   void on_actionByMTime_triggered(bool checked);
@@ -100,6 +116,9 @@ protected Q_SLOTS:
   void on_actionCaseSensitive_triggered(bool checked);
   void on_actionFilter_triggered(bool checked);
 
+  void on_actionLocationBar_triggered(bool checked);
+  void on_actionPathButtons_triggered(bool checked);
+
   void on_actionApplications_triggered();
   void on_actionComputer_triggered();
   void on_actionTrash_triggered();
@@ -120,6 +139,7 @@ protected Q_SLOTS:
   void onTabBarCurrentChanged(int index);
   void onTabBarTabMoved(int from, int to);
 
+  void focusFilterBar();
   void onFilterStringChanged(QString str);
 
   void onShortcutPrevTab();
@@ -140,28 +160,48 @@ protected Q_SLOTS:
   void onSidePaneCreateNewFolderRequested(FmPath* path);
   void onSidePaneModeChanged(Fm::SidePane::Mode mode);
   void onSplitterMoved(int pos, int index);
+  void onResetFocus();
 
   void onBackForwardContextMenu(QPoint pos);
 
+  void tabContextMenu(const QPoint& pos);
+  void closeLeftTabs();
+  void closeRightTabs();
+  void closeOtherTabs() {
+    closeLeftTabs();
+    closeRightTabs();
+  }
+  void focusPathEntry();
+  void toggleMenuBar(bool checked);
+
 protected:
-  // void changeEvent( QEvent * event);
+  bool event(QEvent* event) override;
+  void changeEvent(QEvent *event) override;
   void closeTab(int index);
-  virtual void resizeEvent(QResizeEvent *event);
-  virtual void closeEvent(QCloseEvent *event);
+  virtual void resizeEvent(QResizeEvent *event) override;
+  virtual void closeEvent(QCloseEvent *event) override;
 
 private:
-  static void onBookmarksChanged(FmBookmarks* bookmarks, MainWindow* pThis);
+  static void onBookmarksChanged(FmBookmarks* bookmarks_, MainWindow* pThis);
   void loadBookmarksMenu();
   void updateUIForCurrentPage();
   void updateViewMenuForCurrentPage();
   void updateStatusBarForCurrentPage();
+  void setRTLIcons(bool isRTL);
+  void createPathBar(bool usePathButtons);
 
 private:
   Ui::MainWindow ui;
-  QLineEdit* pathEntry;
-  QLabel* fsInfoLabel;
-  FmBookmarks* bookmarks;
+  Fm::PathEdit* pathEntry_;
+  Fm::PathBar* pathBar_;
+  QLabel* fsInfoLabel_;
+  Fm::Bookmarks bookmarks_;
   Launcher fileLauncher_;
+  int rightClickIndex_;
+  bool updatingViewMenu_;
+  QAction* menuSep_;
+
+  static MainWindow* lastActive_;
 };
 
 }
