@@ -26,9 +26,9 @@
 #include <QStringBuilder>
 #include <QSettings>
 
-#include "folderview.h"
+#include <libfm-qt/folderview.h>
 
-using namespace PCManFM;
+namespace PCManFM {
 
 static int bigIconSizes[] = {96, 72, 64, 48, 36, 32, 24, 20};
 static int smallIconSizes[] = {48, 36, 32, 24, 20, 16, 12};
@@ -45,13 +45,7 @@ PreferencesDialog::PreferencesDialog (QString activePage, QWidget* parent):
 
   initFromSettings();
 
-  if(!activePage.isEmpty()) {
-    QWidget* page = findChild<QWidget*>(activePage + "Page");
-    if(page) {
-      int index = ui.stackedWidget->indexOf(page);
-      ui.listWidget->setCurrentRow(index);
-    }
-  }
+  selectPage(activePage);
   adjustSize();
 }
 
@@ -118,6 +112,10 @@ void PreferencesDialog::initIconThemes(Settings& settings) {
     ui.iconThemeLabel->hide();
     ui.iconTheme->hide();
   }
+
+  ui.hMargin->setValue(settings.folderViewCellMargins().width());
+  ui.vMargin->setValue(settings.folderViewCellMargins().height());
+  connect(ui.lockMargins, &QAbstractButton::clicked, this, &PreferencesDialog::lockMargins);
 }
 
 void PreferencesDialog::initArchivers(Settings& settings) {
@@ -134,14 +132,13 @@ void PreferencesDialog::initArchivers(Settings& settings) {
 void PreferencesDialog::initDisplayPage(Settings& settings) {
     initIconThemes(settings);
     // icon sizes
-    int i;
-    for(i = 0; i < G_N_ELEMENTS(bigIconSizes); ++i) {
+    for(std::size_t i = 0; i < G_N_ELEMENTS(bigIconSizes); ++i) {
       int size = bigIconSizes[i];
       ui.bigIconSize->addItem(QString("%1 x %1").arg(size), size);
       if(settings.bigIconSize() == size)
         ui.bigIconSize->setCurrentIndex(i);
     }
-    for(i = 0; i < G_N_ELEMENTS(smallIconSizes); ++i) {
+    for(std::size_t i = 0; i < G_N_ELEMENTS(smallIconSizes); ++i) {
       int size = smallIconSizes[i];
       QString text = QString("%1 x %1").arg(size);
       ui.smallIconSize->addItem(text, size);
@@ -152,7 +149,7 @@ void PreferencesDialog::initDisplayPage(Settings& settings) {
       if(settings.sidePaneIconSize() == size)
         ui.sidePaneIconSize->setCurrentIndex(i);
     }
-    for(i = 0; i < G_N_ELEMENTS(thumbnailIconSizes); ++i) {
+    for(std::size_t i = 0; i < G_N_ELEMENTS(thumbnailIconSizes); ++i) {
       int size = thumbnailIconSizes[i];
       ui.thumbnailIconSize->addItem(QString("%1 x %1").arg(size), size);
       if(settings.thumbnailIconSize() == size)
@@ -172,6 +169,7 @@ void PreferencesDialog::initDisplayPage(Settings& settings) {
 
 void PreferencesDialog::initUiPage(Settings& settings) {
   ui.alwaysShowTabs->setChecked(settings.alwaysShowTabs());
+  ui.fullWidthTabbar->setChecked(settings.fullWidthTabBar());
   ui.showTabClose->setChecked(settings.showTabClose());
   ui.rememberWindowSize->setChecked(settings.rememberWindowSize());
   ui.fixedWindowWidth->setValue(settings.fixedWindowWidth());
@@ -188,7 +186,7 @@ void PreferencesDialog::initBehaviorPage(Settings& settings) {
   ui.bookmarkOpenMethod->setCurrentIndex(settings.bookmarkOpenMethod());
 
   ui.viewMode->addItem(tr("Icon View"), (int)Fm::FolderView::IconMode);
-  ui.viewMode->addItem(tr("Compact Icon View"), (int)Fm::FolderView::CompactMode);
+  ui.viewMode->addItem(tr("Compact View"), (int)Fm::FolderView::CompactMode);
   ui.viewMode->addItem(tr("Thumbnail View"), (int)Fm::FolderView::ThumbnailMode);
   ui.viewMode->addItem(tr("Detailed List View"), (int)Fm::FolderView::DetailedListMode);
   const Fm::FolderView::ViewMode modes[] = {
@@ -197,7 +195,7 @@ void PreferencesDialog::initBehaviorPage(Settings& settings) {
     Fm::FolderView::ThumbnailMode,
     Fm::FolderView::DetailedListMode
   };
-  for(int i = 0; i < G_N_ELEMENTS(modes); ++i) {
+  for(std::size_t i = 0; i < G_N_ELEMENTS(modes); ++i) {
     if(modes[i] == settings.viewMode()) {
       ui.viewMode->setCurrentIndex(i);
       break;
@@ -289,10 +287,12 @@ void PreferencesDialog::applyDisplayPage(Settings& settings) {
   settings.setBackupAsHidden(ui.backupAsHidden->isChecked());
   settings.setShowFullNames(ui.showFullNames->isChecked());
   settings.setShadowHidden(ui.shadowHidden->isChecked());
+  settings.setFolderViewCellMargins(QSize(ui.hMargin->value(), ui.vMargin->value()));
 }
 
 void PreferencesDialog::applyUiPage(Settings& settings) {
   settings.setAlwaysShowTabs(ui.alwaysShowTabs->isChecked());
+  settings.setFullWidthTabBar(ui.fullWidthTabbar->isChecked());
   settings.setShowTabClose(ui.showTabClose->isChecked());
   settings.setRememberWindowSize(ui.rememberWindowSize->isChecked());
   settings.setFixedWindowWidth(ui.fixedWindowWidth->value());
@@ -362,3 +362,24 @@ void PreferencesDialog::accept() {
   QDialog::accept();
 }
 
+void PreferencesDialog::selectPage(QString name) {
+  if(!name.isEmpty()) {
+    QWidget* page = findChild<QWidget*>(name + "Page");
+    if(page) {
+      int index = ui.stackedWidget->indexOf(page);
+      ui.listWidget->setCurrentRow(index);
+    }
+  }
+}
+
+void PreferencesDialog::lockMargins(bool lock) {
+  ui.vMargin->setDisabled(lock);
+  if(lock) {
+    ui.vMargin->setValue(ui.hMargin->value());
+    connect(ui.hMargin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), ui.vMargin, &QSpinBox::setValue);
+  }
+  else
+    disconnect(ui.hMargin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), ui.vMargin, &QSpinBox::setValue);
+}
+
+} // namespace PCManFM
