@@ -25,10 +25,11 @@
 #include <QSettings>
 #include <QApplication>
 #include "desktopwindow.h"
-#include "utilities.h"
-// #include <QDesktopServices>
+#include <libfm-qt/utilities.h>
+#include <libfm-qt/folderconfig.h>
+#include <QStandardPaths>
 
-using namespace PCManFM;
+namespace PCManFM {
 
 inline static const char* bookmarkOpenMethodToString(OpenDirTargetType value);
 inline static OpenDirTargetType bookmarkOpenMethodFromString(const QString str);
@@ -65,10 +66,12 @@ Settings::Settings():
   desktopBgColor_(),
   desktopFgColor_(),
   desktopShadowColor_(),
+  desktopIconSize_(48),
   showWmMenu_(false),
   desktopShowHidden_(false),
   desktopSortOrder_(Qt::AscendingOrder),
-  desktopSortColumn_(Fm::FolderModel::ColumnFileName),
+  desktopSortColumn_(Fm::FolderModel::ColumnFileMTime),
+  desktopSortFolderFirst_(true),
   alwaysShowTabs_(true),
   showTabClose_(true),
   rememberWindowSize_(true),
@@ -79,12 +82,16 @@ Settings::Settings():
   lastWindowMaximized_(false),
   splitterPos_(120),
   sidePaneMode_(Fm::SidePane::ModePlaces),
+  showMenuBar_(true),
+  fullWidthTabBar_(true),
   viewMode_(Fm::FolderView::IconMode),
   showHidden_(false),
   sortOrder_(Qt::AscendingOrder),
   sortColumn_(Fm::FolderModel::ColumnFileName),
   sortFolderFirst_(true),
+  sortCaseSensitive_(false),
   showFilter_(false),
+  pathBarButtons_(true),
   // settings for use with libfm
   singleClick_(false),
   autoSelectionDelay_(600),
@@ -96,22 +103,39 @@ Settings::Settings():
   showThumbnails_(true),
   archiver_(),
   siUnit_(false),
+  placesHome_(true),
+  placesDesktop_(true),
+  placesApplications_(true),
+  placesTrash_(true),
+  placesRoot_(true),
+  placesComputer_(true),
+  placesNetwork_(true),
   bigIconSize_(48),
   smallIconSize_(24),
   sidePaneIconSize_(24),
-  thumbnailIconSize_(128) {
+  thumbnailIconSize_(128),
+  folderViewCellMargins_(QSize(3, 3)),
+  desktopCellMargins_(QSize(3, 1)) {
 }
 
 Settings::~Settings() {
 
 }
 
-QString Settings::profileDir(QString profile, bool useFallback) {
-  // NOTE: it's a shame that QDesktopServices does not handle XDG_CONFIG_HOME
-  // try user-specific config file first
-  QString dirName = QLatin1String(qgetenv("XDG_CONFIG_HOME"));
+QString Settings::xdgUserConfigDir() {
+  QString dirName;
+  // WARNING: Don't use XDG_CONFIG_HOME with root because it might
+  // give the user config directory if gksu-properties is set to su.
+  if(geteuid() != 0)  // non-root user
+    dirName = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
   if (dirName.isEmpty())
     dirName = QDir::homePath() % QLatin1String("/.config");
+  return dirName;
+}
+
+QString Settings::profileDir(QString profile, bool useFallback) {
+  // try user-specific config file first
+  QString dirName = xdgUserConfigDir();
   dirName = dirName % "/pcmanfm-qt/" % profile;
   QDir dir(dirName);
 
@@ -148,15 +172,15 @@ bool Settings::loadFile(QString filePath) {
   if(fallbackIconThemeName_.isEmpty()) {
     // FIXME: we should choose one from installed icon themes or get
     // the value from XSETTINGS instead of hard code a fallback value.
-    fallbackIconThemeName_ = "elementary"; // fallback icon theme name
+    fallbackIconThemeName_ = "oxygen"; // fallback icon theme name
   }
-  suCommand_ = settings.value("SuCommand", "gksu %s").toString();
+  suCommand_ = settings.value("SuCommand", "lxqt-sudo %s").toString();
   setTerminal(settings.value("Terminal", "xterm").toString());
   setArchiver(settings.value("Archiver", "file-roller").toString());
   setSiUnit(settings.value("SIUnit", false).toBool());
 
   setOnlyUserTemplates(settings.value("OnlyUserTemplates", false).toBool());
-  setTemplateTypeOnce(settings.value("OemplateTypeOnce", false).toBool());
+  setTemplateTypeOnce(settings.value("TemplateTypeOnce", false).toBool());
   setTemplateRunApp(settings.value("TemplateRunApp", false).toBool());
 
   settings.endGroup();
@@ -185,11 +209,16 @@ bool Settings::loadFile(QString filePath) {
     desktopFont_.fromString(settings.value("Font").toString());
   else
     desktopFont_ = QApplication::font();
+  desktopIconSize_ = settings.value("DesktopIconSize", 48).toInt();
   showWmMenu_ = settings.value("ShowWmMenu", false).toBool();
   desktopShowHidden_ = settings.value("ShowHidden", false).toBool();
 
   desktopSortOrder_ = sortOrderFromString(settings.value("SortOrder").toString());
   desktopSortColumn_ = sortColumnFromString(settings.value("SortColumn").toString());
+  desktopSortFolderFirst_ = settings.value("SortFolderFirst", true).toBool();
+
+  desktopCellMargins_ = (settings.value("DesktopCellMargins", QSize(3, 1)).toSize()
+                         .expandedTo(QSize(0, 0))).boundedTo(QSize(48, 48));
   settings.endGroup();
 
   settings.beginGroup("Volume");
@@ -211,6 +240,7 @@ bool Settings::loadFile(QString filePath) {
   sortOrder_ = sortOrderFromString(settings.value("SortOrder").toString());
   sortColumn_ = sortColumnFromString(settings.value("SortColumn").toString());
   sortFolderFirst_ = settings.value("SortFolderFirst", true).toBool();
+  sortCaseSensitive_ = settings.value("SortCaseSensitive", false).toBool();
   showFilter_ = settings.value("ShowFilter", false).toBool();
 
   setBackupAsHidden(settings.value("BackupAsHidden", false).toBool());
@@ -222,6 +252,19 @@ bool Settings::loadFile(QString filePath) {
   smallIconSize_ = settings.value("SmallIconSize", 24).toInt();
   sidePaneIconSize_ = settings.value("SidePaneIconSize", 24).toInt();
   thumbnailIconSize_ = settings.value("ThumbnailIconSize", 128).toInt();
+
+  folderViewCellMargins_ = (settings.value("FolderViewCellMargins", QSize(3, 3)).toSize()
+                            .expandedTo(QSize(0, 0))).boundedTo(QSize(48, 48));
+  settings.endGroup();
+
+  settings.beginGroup("Places");
+  placesHome_ = settings.value("PlacesHome", true).toBool();
+  placesDesktop_ = settings.value("PlacesDesktop", true).toBool();
+  placesApplications_ = settings.value("PlacesApplications", true).toBool();
+  placesTrash_ = settings.value("PlacesTrash", true).toBool();
+  placesRoot_ = settings.value("PlacesRoot", true).toBool();
+  placesComputer_ = settings.value("PlacesComputer", true).toBool();
+  placesNetwork_ = settings.value("PlacesNetwork", true).toBool();
   settings.endGroup();
 
   settings.beginGroup("Window");
@@ -235,6 +278,9 @@ bool Settings::loadFile(QString filePath) {
   showTabClose_ = settings.value("ShowTabClose", true).toBool();
   splitterPos_ = settings.value("SplitterPos", 150).toInt();
   sidePaneMode_ = sidePaneModeFromString(settings.value("SidePaneMode").toString());
+  showMenuBar_ = settings.value("ShowMenuBar", true).toBool();
+  fullWidthTabBar_ = settings.value("FullWidthTabBar", true).toBool();
+  pathBarButtons_ = settings.value("PathBarButtons", true).toBool();
   settings.endGroup();
 
   return true;
@@ -251,7 +297,7 @@ bool Settings::saveFile(QString filePath) {
   settings.setValue("SIUnit", siUnit_);
 
   settings.setValue("OnlyUserTemplates", onlyUserTemplates_);
-  settings.setValue("OemplateTypeOnce", templateTypeOnce_);
+  settings.setValue("TemplateTypeOnce", templateTypeOnce_);
   settings.setValue("TemplateRunApp", templateRunApp_);
 
   settings.endGroup();
@@ -277,10 +323,13 @@ bool Settings::saveFile(QString filePath) {
   settings.setValue("FgColor", desktopFgColor_.name());
   settings.setValue("ShadowColor", desktopShadowColor_.name());
   settings.setValue("Font", desktopFont_.toString());
+  settings.setValue("DesktopIconSize", desktopIconSize_);
   settings.setValue("ShowWmMenu", showWmMenu_);
   settings.setValue("ShowHidden", desktopShowHidden_);
   settings.setValue("SortOrder", sortOrderToString(desktopSortOrder_));
   settings.setValue("SortColumn", sortColumnToString(desktopSortColumn_));
+  settings.setValue("SortFolderFirst", desktopSortFolderFirst_);
+  settings.setValue("DesktopCellMargins", desktopCellMargins_);
   settings.endGroup();
 
   settings.beginGroup("Volume");
@@ -302,6 +351,7 @@ bool Settings::saveFile(QString filePath) {
   settings.setValue("SortOrder", sortOrderToString(sortOrder_));
   settings.setValue("SortColumn", sortColumnToString(sortColumn_));
   settings.setValue("SortFolderFirst", sortFolderFirst_);
+  settings.setValue("SortCaseSensitive", sortCaseSensitive_);
   settings.setValue("ShowFilter", showFilter_);
 
   settings.setValue("BackupAsHidden", backupAsHidden_);
@@ -313,6 +363,18 @@ bool Settings::saveFile(QString filePath) {
   settings.setValue("SmallIconSize", smallIconSize_);
   settings.setValue("SidePaneIconSize", sidePaneIconSize_);
   settings.setValue("ThumbnailIconSize", thumbnailIconSize_);
+
+  settings.setValue("FolderViewCellMargins", folderViewCellMargins_);
+  settings.endGroup();
+
+  settings.beginGroup("Places");
+  settings.setValue("PlacesHome", placesHome_);
+  settings.setValue("PlacesDesktop", placesDesktop_);
+  settings.setValue("PlacesApplications", placesApplications_);
+  settings.setValue("PlacesTrash", placesTrash_);
+  settings.setValue("PlacesRoot", placesRoot_);
+  settings.setValue("PlacesComputer", placesComputer_);
+  settings.setValue("PlacesNetwork", placesNetwork_);
   settings.endGroup();
 
   settings.beginGroup("Window");
@@ -326,7 +388,14 @@ bool Settings::saveFile(QString filePath) {
   settings.setValue("ShowTabClose", showTabClose_);
   settings.setValue("SplitterPos", splitterPos_);
   settings.setValue("SidePaneMode", sidePaneModeToString(sidePaneMode_));
+  settings.setValue("ShowMenuBar", showMenuBar_);
+  settings.setValue("FullWidthTabBar", fullWidthTabBar_);
+  settings.setValue("PathBarButtons", pathBarButtons_);
   settings.endGroup();
+
+  // save per-folder settings
+  Fm::FolderConfig::saveCache();
+
   return true;
 }
 
@@ -458,6 +527,9 @@ static const char* wallpaperModeToString(int value) {
     case DesktopWindow::WallpaperTile:
       ret = "tile";
       break;
+    case DesktopWindow::WallpaperZoom:
+      ret = "zoom";
+      break;
   }
   return ret;
 }
@@ -472,6 +544,8 @@ static int wallpaperModeFromString(const QString str) {
     ret = DesktopWindow::WallpaperCenter;
   else if(str == "tile")
     ret = DesktopWindow::WallpaperTile;
+  else if(str == "zoom")
+    ret = DesktopWindow::WallpaperZoom;
   else
     ret = DesktopWindow::WallpaperNone;
   return ret;
@@ -511,4 +585,86 @@ void Settings::setTerminal(QString terminalCommand) {
     g_free(fm_config->terminal);
     fm_config->terminal = g_strdup(terminal_.toLocal8Bit().constData());
     g_signal_emit_by_name(fm_config, "changed::terminal");
+}
+
+
+// per-folder settings
+FolderSettings Settings::loadFolderSettings(Fm::Path path) const {
+  FolderSettings settings;
+  Fm::FolderConfig cfg(path);
+  // set defaults
+  settings.setSortOrder(sortOrder());
+  settings.setSortColumn(sortColumn());
+  settings.setViewMode(viewMode());
+  settings.setShowHidden(showHidden());
+  settings.setSortFolderFirst(sortFolderFirst());
+  settings.setSortCaseSensitive(sortCaseSensitive());
+  // columns?
+  if(!cfg.isEmpty()) {
+    // load folder-specific settings
+    settings.setCustomized(true);
+
+    char* str;
+    // load sorting
+    str = cfg.getString("SortOrder");
+    if(str != nullptr) {
+      settings.setSortOrder(sortOrderFromString(str));
+      g_free(str);
+    }
+
+    str = cfg.getString("SortColumn");
+    if(str != nullptr) {
+      settings.setSortColumn(sortColumnFromString(str));
+      g_free(str);
+    }
+
+    str = cfg.getString("ViewMode");
+    if(str != nullptr) {
+      // set view mode
+      settings.setViewMode(viewModeFromString(str));
+      g_free(str);
+    }
+
+    gboolean show_hidden;
+    if(cfg.getBoolean("ShowHidden", &show_hidden)) {
+      settings.setShowHidden(show_hidden);
+    }
+
+    gboolean folder_first;
+    if(cfg.getBoolean("SortFolderFirst", &folder_first)) {
+      settings.setSortFolderFirst(folder_first);
+    }
+
+    gboolean case_sensitive;
+    if(cfg.getBoolean("SortCaseSensitive", &case_sensitive)) {
+      settings.setSortCaseSensitive(case_sensitive);
+    }
   }
+  return settings;
+}
+
+void Settings::saveFolderSettings(Fm::Path path, const FolderSettings& settings) {
+  if(!path.isNull()) {
+    // ensure that we have the libfm dir
+    QString dirName = xdgUserConfigDir() % QStringLiteral("/libfm");
+    QDir().mkpath(dirName);  // if libfm config dir does not exist, create it
+
+    Fm::FolderConfig cfg(path);
+    cfg.setString("SortOrder", sortOrderToString(settings.sortOrder()));
+    cfg.setString("SortColumn", sortColumnToString(settings.sortColumn()));
+    cfg.setString("ViewMode", viewModeToString(settings.viewMode()));
+    cfg.setBoolean("ShowHidden", settings.showHidden());
+    cfg.setBoolean("SortFolderFirst", settings.sortFolderFirst());
+    cfg.setBoolean("SortCaseSensitive", settings.sortCaseSensitive());
+  }
+}
+
+void Settings::clearFolderSettings(Fm::Path path) const {
+  if(!path.isNull()) {
+    Fm::FolderConfig cfg(path);
+    cfg.purge();
+  }
+}
+
+
+} // namespace PCManFM
