@@ -23,11 +23,11 @@
 
 #include <QObject>
 #include <libfm/fm.h>
-#include "folderview.h"
-#include "foldermodel.h"
+#include <libfm-qt/folderview.h>
+#include <libfm-qt/foldermodel.h>
 #include "desktopwindow.h"
-#include "sidepane.h"
-#include "thumbnailloader.h"
+#include <libfm-qt/sidepane.h>
+#include <libfm-qt/thumbnailloader.h>
 
 namespace PCManFM {
 
@@ -38,6 +38,86 @@ enum OpenDirTargetType {
     OpenInLastActiveWindow
 };
 
+class FolderSettings {
+public:
+  FolderSettings():
+    isCustomized_(false),
+    sortOrder_(Qt::AscendingOrder),
+    sortColumn_(Fm::FolderModel::ColumnFileName),
+    viewMode_(Fm::FolderView::IconMode),
+    showHidden_(false),
+    sortFolderFirst_(true),
+    sortCaseSensitive_(true) {
+  }
+
+  bool isCustomized() const {
+    return isCustomized_;
+  }
+
+  void setCustomized(bool value) {
+    isCustomized_ = value;
+  }
+
+  Qt::SortOrder sortOrder() const {
+    return sortOrder_;
+  }
+
+  void setSortOrder(Qt::SortOrder value) {
+    sortOrder_ = value;
+  }
+
+  Fm::FolderModel::ColumnId sortColumn() const {
+    return sortColumn_;
+  }
+
+  void setSortColumn(Fm::FolderModel::ColumnId value) {
+    sortColumn_ = value;
+  }
+
+  Fm::FolderView::ViewMode viewMode() const {
+    return viewMode_;
+  }
+
+  void setViewMode(Fm::FolderView::ViewMode value) {
+    viewMode_ = value;
+  }
+
+  bool sortFolderFirst() const {
+    return sortFolderFirst_;
+  }
+
+  void setSortFolderFirst(bool value) {
+    sortFolderFirst_ = value;
+  }
+
+  bool showHidden() const {
+    return showHidden_;
+  }
+
+  void setShowHidden(bool value) {
+    showHidden_ = value;
+  }
+
+  bool sortCaseSensitive() const {
+    return sortCaseSensitive_;
+  }
+
+  void setSortCaseSensitive(bool value) {
+    sortCaseSensitive_ = value;
+  }
+
+private:
+  bool isCustomized_;
+  Qt::SortOrder sortOrder_;
+  Fm::FolderModel::ColumnId sortColumn_;
+  Fm::FolderView::ViewMode viewMode_;
+  bool showHidden_;
+  bool sortFolderFirst_;
+  bool sortCaseSensitive_;
+  // columns?
+};
+
+
 class Settings : public QObject {
   Q_OBJECT
 public:
@@ -50,6 +130,8 @@ public:
   bool loadFile(QString filePath);
   bool saveFile(QString filePath);
 
+  static QString xdgUserConfigDir();
+
   QString profileDir(QString profile, bool useFallback = false);
 
   // setter/getter functions
@@ -185,6 +267,14 @@ public:
     desktopFont_ = font;
   }
 
+  int desktopIconSize() const {
+    return desktopIconSize_;
+  }
+
+  void setDesktopIconSize(int desktopIconSize) {
+    desktopIconSize_ = desktopIconSize;
+  }
+
   bool showWmMenu() const {
     return showWmMenu_;
   }
@@ -217,6 +307,14 @@ public:
     desktopSortColumn_ = desktopSortColumn;
   }
 
+  bool desktopSortFolderFirst() const {
+    return desktopSortFolderFirst_;
+  }
+
+  void setSesktopSortFolderFirst(bool desktopFolderFirst) {
+    desktopSortFolderFirst_ = desktopFolderFirst;
+  }
+
   bool alwaysShowTabs() const {
     return alwaysShowTabs_;
   }
@@ -306,6 +404,22 @@ public:
     sidePaneMode_ = sidePaneMode;
   }
 
+  bool showMenuBar() const {
+    return showMenuBar_;
+  }
+
+  void setShowMenuBar(bool showMenuBar) {
+    showMenuBar_ = showMenuBar;
+  }
+
+  bool fullWidthTabBar() const {
+    return fullWidthTabBar_;
+  }
+
+  void setFullWidthTabBar(bool fullWith) {
+    fullWidthTabBar_ = fullWith;
+  }
+
   Fm::FolderView::ViewMode viewMode() const {
     return viewMode_;
   }
@@ -322,6 +436,72 @@ public:
     showHidden_ = showHidden;
   }
 
+  bool sortCaseSensitive() const {
+    return sortCaseSensitive_;
+  }
+
+  void setSortCaseSensitive(bool value) {
+    sortCaseSensitive_ = value;
+  }
+
+
+  bool placesHome() const {
+      return placesHome_;
+  }
+
+  void setPlacesHome(bool placesHome) {
+      placesHome_ = placesHome;
+  }
+
+  bool placesDesktop() const {
+      return placesDesktop_;
+  }
+
+  void setPlacesDesktop(bool placesDesktop) {
+      placesDesktop_ = placesDesktop;
+  }
+
+  bool placesApplications() const {
+      return placesApplications_;
+  }
+
+  void setPlacesApplications(bool placesApplications) {
+      placesApplications_ = placesApplications;
+  }
+
+  bool placesTrash() const {
+      return placesTrash_;
+  }
+
+  void setPlacesTrash(bool placesTrash) {
+      placesTrash_ = placesTrash;
+  }
+
+  bool placesRoot() const {
+      return placesRoot_;
+  }
+
+  void setPlacesRoot(bool placesRoot) {
+      placesRoot_ = placesRoot;
+  }
+
+  bool placesComputer() const {
+      return placesComputer_;
+  }
+
+  void setPlacesComputer(bool placesComputer) {
+      placesComputer_ = placesComputer;
+  }
+
+  bool placesNetwork() const {
+      return placesNetwork_;
+  }
+
+  void setPlacesNetwork(bool placesNetwork) {
+      placesNetwork_ = placesNetwork;
+  }
+
+
   Qt::SortOrder sortOrder() const {
     return sortOrder_;
   }
@@ -354,6 +534,14 @@ public:
     showFilter_ = value;
   }
 
+  bool pathBarButtons() const {
+    return pathBarButtons_;
+  }
+
+  void setPathBarButtons(bool value) {
+    pathBarButtons_ = value;
+  }
+
   // settings for use with libfm
   bool singleClick() const {
     return singleClick_;
@@ -446,6 +634,23 @@ public:
     return thumbnailIconSize_;
   }
 
+  QSize folderViewCellMargins() const {
+    return folderViewCellMargins_;
+  }
+
+  void setFolderViewCellMargins(QSize size) {
+    folderViewCellMargins_ = size;
+  }
+
+  QSize desktopCellMargins() const {
+    return desktopCellMargins_;
+  }
+
+  void setDesktopCellMargins(QSize size) {
+    desktopCellMargins_ = size;
+  }
+
+
   bool showThumbnails() {
     return showThumbnails_;
   }
@@ -536,6 +741,13 @@ public:
     fm_config->template_run_app = templateRunApp_;
   }
 
+  // per-folder settings
+  FolderSettings loadFolderSettings(Fm::Path path) const;
+
+  void saveFolderSettings(Fm::Path path, const FolderSettings &settings);
+
+  void clearFolderSettings(Fm::Path path) const;
+
 private:
   QString profileName_;
   bool supportTrash_;
@@ -558,11 +770,13 @@ private:
   QColor desktopFgColor_;
   QColor desktopShadowColor_;
   QFont desktopFont_;
+  int desktopIconSize_;
   bool showWmMenu_;
 
   bool desktopShowHidden_;
   Qt::SortOrder desktopSortOrder_;
   Fm::FolderModel::ColumnId desktopSortColumn_;
+  bool desktopSortFolderFirst_;
 
   bool alwaysShowTabs_;
   bool showTabClose_;
@@ -574,13 +788,17 @@ private:
   bool lastWindowMaximized_;
   int splitterPos_;
   Fm::SidePane::Mode sidePaneMode_;
+  bool showMenuBar_;
+  bool fullWidthTabBar_;
 
   Fm::FolderView::ViewMode viewMode_;
   bool showHidden_;
   Qt::SortOrder sortOrder_;
   Fm::FolderModel::ColumnId sortColumn_;
   bool sortFolderFirst_;
+  bool sortCaseSensitive_;
   bool showFilter_;
+  bool pathBarButtons_;
 
   // settings for use with libfm
   bool singleClick_;
@@ -599,6 +817,14 @@ private:
   bool showFullNames_;
   bool shadowHidden_;
 
+  bool placesHome_;
+  bool placesDesktop_;
+  bool placesApplications_;
+  bool placesTrash_;
+  bool placesRoot_;
+  bool placesComputer_;
+  bool placesNetwork_;
+
   int bigIconSize_;
   int smallIconSize_;
   int sidePaneIconSize_;
@@ -607,6 +833,9 @@ private:
   bool onlyUserTemplates_;
   bool templateTypeOnce_;
   bool templateRunApp_;
+
+  QSize folderViewCellMargins_;
+  QSize desktopCellMargins_;
 };
 
 }
