@@ -19,9 +19,8 @@
 
 
 #include "view.h"
-#include "filemenu.h"
-#include "foldermenu.h"
-#include "filelauncher.h"
+#include <libfm-qt/filemenu.h>
+#include <libfm-qt/foldermenu.h>
 #include "application.h"
 #include "settings.h"
 #include "application.h"
@@ -29,7 +28,7 @@
 #include "launcher.h"
 #include <QAction>
 
-using namespace PCManFM;
+namespace PCManFM {
 
 View::View(Fm::FolderView::ViewMode _mode, QWidget* parent):
   Fm::FolderView(_mode, parent) {
@@ -71,8 +70,8 @@ void View::onOpenInTerminal() {
   Application* app = static_cast<Application*>(qApp);
   Fm::FileMenu* menu = static_cast<Fm::FileMenu*>(sender()->parent());
   for(GList* l = fm_file_info_list_peek_head_link(menu->files()); l; l = l->next) {
-    FmFileInfo* file = FM_FILE_INFO(l->data);
-    app->openFolderInTerminal(fm_file_info_get_path(file));
+    Fm::FileInfo file = FM_FILE_INFO(l->data);
+    app->openFolderInTerminal(file.getPath());
   }
 }
 
@@ -88,37 +87,45 @@ void View::prepareFileMenu(Fm::FileMenu* menu) {
 
   // add some more menu items for dirs
   bool all_native = true;
-  FmFileInfoList* files = menu->files();
-  for(GList* l = fm_file_info_list_peek_head_link(files); l; l = l->next) {
-    FmFileInfo* fi = FM_FILE_INFO(l->data);
-    if(!fm_file_info_is_dir(fi))
-      return; /* actions are valid only if all selected are directories */
-    else if(!fm_file_info_is_native(fi))
+  bool all_directory = true;
+  Fm::FileInfoList files = menu->files();
+  for(GList* l = files.peekHeadLink(); l; l = l->next) {
+    Fm::FileInfo fi = FM_FILE_INFO(l->data);
+    if(!fi.isDir())
+      all_directory = false;
+    else if(fi.isDir() && !fi.isNative())
       all_native = false;
   }
 
-  // hide "Open with" for selected dirs
-  menu->openWithMenuAction()->setVisible(false);
-
-  QAction* action = new QAction(QIcon::fromTheme("window-new"), tr("Open in New T&ab"), menu);
-  connect(action, &QAction::triggered, this, &View::onNewTab);
-  menu->insertAction(menu->separator1(), action);
-
-  action = new QAction(QIcon::fromTheme("window-new"), tr("Open in New Win&dow"), menu);
-  connect(action, &QAction::triggered, this, &View::onNewWindow);
-  menu->insertAction(menu->separator1(), action);
-
-  // TODO: add search
-  // action = menu->addAction(_("Search"));
-  if(all_native) {
-    action = new QAction(QIcon::fromTheme("utilities-terminal"), tr("Open in Termina&l"), menu);
-    connect(action, &QAction::triggered, this, &View::onOpenInTerminal);
+  if (all_directory)
+  {
+    QAction* action = new QAction(QIcon::fromTheme("window-new"), tr("Open in New T&ab"), menu);
+    connect(action, &QAction::triggered, this, &View::onNewTab);
     menu->insertAction(menu->separator1(), action);
+
+    action = new QAction(QIcon::fromTheme("window-new"), tr("Open in New Win&dow"), menu);
+    connect(action, &QAction::triggered, this, &View::onNewWindow);
+    menu->insertAction(menu->separator1(), action);
+
+    // TODO: add search
+    // action = menu->addAction(_("Search"));
+
+    if(all_native)
+    {
+      action = new QAction(QIcon::fromTheme("utilities-terminal"), tr("Open in Termina&l"), menu);
+      connect(action, &QAction::triggered, this, &View::onOpenInTerminal);
+      menu->insertAction(menu->separator1(), action);
+    }
+  }
+  else {
+    if(menu->pasteAction()) // NULL for trash
+      menu->pasteAction()->setVisible(false);
+    if(menu->createAction())
+      menu->createAction()->setVisible(false);
   }
 }
 
 void View::prepareFolderMenu(Fm::FolderMenu* menu) {
-
 }
 
 void View::updateFromSettings(Settings& settings) {
@@ -128,6 +135,8 @@ void View::updateFromSettings(Settings& settings) {
   setIconSize(Fm::FolderView::ThumbnailMode, QSize(settings.thumbnailIconSize(), settings.thumbnailIconSize()));
   setIconSize(Fm::FolderView::DetailedListMode, QSize(settings.smallIconSize(), settings.smallIconSize()));
 
+  setMargins(settings.folderViewCellMargins());
+
   setAutoSelectionDelay(settings.autoSelectionDelay());
 
   Fm::ProxyFolderModel* proxyModel = model();
@@ -136,3 +145,4 @@ void View::updateFromSettings(Settings& settings) {
   }
 }
 
+} // namespace PCManFM
